-- The layer8 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer8 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic_vector(2 downto 0);
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO256 : in std_logic_vector(7 downto 0);
  CO256_dly2 : in std_logic_vector(7 downto 0);
  X_RD_DATA : in std_logic_vector(1279 downto 0); 
  ReluAll : out std_logic_vector(2559 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_layer8;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer8 is

component dotpro_w8
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is1 : in std_logic;
  CO256is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(13 downto 0) );  
end component;

component ROM_W8
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(11 downto 0);
          DATA : Out std_logic_vector(1535 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_00, mac_01, mac_02, mac_03, mac_04, mac_05, mac_06, mac_07, mac_08, mac_09 : std_logic_vector(13 downto 0);
signal mac_10, mac_11, mac_12, mac_13, mac_14, mac_15, mac_16, mac_17, mac_18, mac_19 : std_logic_vector(13 downto 0);
signal mac_20, mac_21, mac_22, mac_23, mac_24, mac_25, mac_26, mac_27, mac_28, mac_29 : std_logic_vector(13 downto 0);
signal mac_30, mac_31, mac_32, mac_33, mac_34, mac_35, mac_36, mac_37, mac_38, mac_39 : std_logic_vector(13 downto 0);
signal mac_40, mac_41, mac_42, mac_43, mac_44, mac_45, mac_46, mac_47, mac_48, mac_49 : std_logic_vector(13 downto 0);
signal mac_50, mac_51, mac_52, mac_53, mac_54, mac_55, mac_56, mac_57, mac_58, mac_59 : std_logic_vector(13 downto 0);
signal mac_60, mac_61, mac_62, mac_63 : std_logic_vector(13 downto 0);

signal truncate_00, truncate_01, truncate_02, truncate_03, truncate_04, truncate_05, truncate_06, truncate_07, truncate_08, truncate_09 : std_logic_vector(6 downto 0);
signal truncate_10, truncate_11, truncate_12, truncate_13, truncate_14, truncate_15, truncate_16, truncate_17, truncate_18, truncate_19 : std_logic_vector(6 downto 0);
signal truncate_20, truncate_21, truncate_22, truncate_23, truncate_24, truncate_25, truncate_26, truncate_27, truncate_28, truncate_29 : std_logic_vector(6 downto 0);
signal truncate_30, truncate_31, truncate_32, truncate_33, truncate_34, truncate_35, truncate_36, truncate_37, truncate_38, truncate_39 : std_logic_vector(6 downto 0);
signal truncate_40, truncate_41, truncate_42, truncate_43, truncate_44, truncate_45, truncate_46, truncate_47, truncate_48, truncate_49 : std_logic_vector(6 downto 0);
signal truncate_50, truncate_51, truncate_52, truncate_53, truncate_54, truncate_55, truncate_56, truncate_57, truncate_58, truncate_59 : std_logic_vector(6 downto 0);
signal truncate_60, truncate_61, truncate_62, truncate_63 : std_logic_vector(6 downto 0);

signal Relu_00, Relu_01, Relu_02, Relu_03, Relu_04, Relu_05, Relu_06, Relu_07, Relu_08, Relu_09 : std_logic_vector(4 downto 0);
signal Relu_10, Relu_11, Relu_12, Relu_13, Relu_14, Relu_15, Relu_16, Relu_17, Relu_18, Relu_19 : std_logic_vector(4 downto 0);
signal Relu_20, Relu_21, Relu_22, Relu_23, Relu_24, Relu_25, Relu_26, Relu_27, Relu_28, Relu_29 : std_logic_vector(4 downto 0);
signal Relu_30, Relu_31, Relu_32, Relu_33, Relu_34, Relu_35, Relu_36, Relu_37, Relu_38, Relu_39 : std_logic_vector(4 downto 0);
signal Relu_40, Relu_41, Relu_42, Relu_43, Relu_44, Relu_45, Relu_46, Relu_47, Relu_48, Relu_49 : std_logic_vector(4 downto 0);
signal Relu_50, Relu_51, Relu_52, Relu_53, Relu_54, Relu_55, Relu_56, Relu_57, Relu_58, Relu_59 : std_logic_vector(4 downto 0);
signal Relu_60, Relu_61, Relu_62, Relu_63 : std_logic_vector(4 downto 0);

signal outCh_000, outCh_001, outCh_002, outCh_003, outCh_004, outCh_005, outCh_006, outCh_007, outCh_008, outCh_009 : std_logic_vector(4 downto 0);

signal PixRomData : std_logic_vector(4 downto 0);
signal W8Addr, W8Addr_prev : std_logic_vector(11 downto 0);
signal W8Addr_L : std_logic_vector(3 downto 0);
signal W8Addr_H : std_logic_vector(11 downto 0);
signal W8Sel : std_logic_vector(2 downto 0);
signal OutChSel_dly1, OutChSel_dly2 : std_logic_vector(2 downto 0);
signal reg_00_0, reg_00_1, reg_00_2, reg_00_3, reg_00_4, reg_00_5, reg_00_6, reg_00_7: std_logic_vector(4 downto 0);
signal reg_01_0, reg_01_1, reg_01_2, reg_01_3, reg_01_4, reg_01_5, reg_01_6, reg_01_7: std_logic_vector(4 downto 0);
signal reg_02_0, reg_02_1, reg_02_2, reg_02_3, reg_02_4, reg_02_5, reg_02_6, reg_02_7: std_logic_vector(4 downto 0);
signal reg_03_0, reg_03_1, reg_03_2, reg_03_3, reg_03_4, reg_03_5, reg_03_6, reg_03_7: std_logic_vector(4 downto 0);
signal reg_04_0, reg_04_1, reg_04_2, reg_04_3, reg_04_4, reg_04_5, reg_04_6, reg_04_7: std_logic_vector(4 downto 0);
signal reg_05_0, reg_05_1, reg_05_2, reg_05_3, reg_05_4, reg_05_5, reg_05_6, reg_05_7: std_logic_vector(4 downto 0);
signal reg_06_0, reg_06_1, reg_06_2, reg_06_3, reg_06_4, reg_06_5, reg_06_6, reg_06_7: std_logic_vector(4 downto 0);
signal reg_07_0, reg_07_1, reg_07_2, reg_07_3, reg_07_4, reg_07_5, reg_07_6, reg_07_7: std_logic_vector(4 downto 0);
signal reg_08_0, reg_08_1, reg_08_2, reg_08_3, reg_08_4, reg_08_5, reg_08_6, reg_08_7: std_logic_vector(4 downto 0);
signal reg_09_0, reg_09_1, reg_09_2, reg_09_3, reg_09_4, reg_09_5, reg_09_6, reg_09_7: std_logic_vector(4 downto 0);
signal reg_10_0, reg_10_1, reg_10_2, reg_10_3, reg_10_4, reg_10_5, reg_10_6, reg_10_7: std_logic_vector(4 downto 0);
signal reg_11_0, reg_11_1, reg_11_2, reg_11_3, reg_11_4, reg_11_5, reg_11_6, reg_11_7: std_logic_vector(4 downto 0);
signal reg_12_0, reg_12_1, reg_12_2, reg_12_3, reg_12_4, reg_12_5, reg_12_6, reg_12_7: std_logic_vector(4 downto 0);
signal reg_13_0, reg_13_1, reg_13_2, reg_13_3, reg_13_4, reg_13_5, reg_13_6, reg_13_7: std_logic_vector(4 downto 0);
signal reg_14_0, reg_14_1, reg_14_2, reg_14_3, reg_14_4, reg_14_5, reg_14_6, reg_14_7: std_logic_vector(4 downto 0);
signal reg_15_0, reg_15_1, reg_15_2, reg_15_3, reg_15_4, reg_15_5, reg_15_6, reg_15_7: std_logic_vector(4 downto 0);
signal reg_16_0, reg_16_1, reg_16_2, reg_16_3, reg_16_4, reg_16_5, reg_16_6, reg_16_7: std_logic_vector(4 downto 0);
signal reg_17_0, reg_17_1, reg_17_2, reg_17_3, reg_17_4, reg_17_5, reg_17_6, reg_17_7: std_logic_vector(4 downto 0);
signal reg_18_0, reg_18_1, reg_18_2, reg_18_3, reg_18_4, reg_18_5, reg_18_6, reg_18_7: std_logic_vector(4 downto 0);
signal reg_19_0, reg_19_1, reg_19_2, reg_19_3, reg_19_4, reg_19_5, reg_19_6, reg_19_7: std_logic_vector(4 downto 0);
signal reg_20_0, reg_20_1, reg_20_2, reg_20_3, reg_20_4, reg_20_5, reg_20_6, reg_20_7: std_logic_vector(4 downto 0);
signal reg_21_0, reg_21_1, reg_21_2, reg_21_3, reg_21_4, reg_21_5, reg_21_6, reg_21_7: std_logic_vector(4 downto 0);
signal reg_22_0, reg_22_1, reg_22_2, reg_22_3, reg_22_4, reg_22_5, reg_22_6, reg_22_7: std_logic_vector(4 downto 0);
signal reg_23_0, reg_23_1, reg_23_2, reg_23_3, reg_23_4, reg_23_5, reg_23_6, reg_23_7: std_logic_vector(4 downto 0);
signal reg_24_0, reg_24_1, reg_24_2, reg_24_3, reg_24_4, reg_24_5, reg_24_6, reg_24_7: std_logic_vector(4 downto 0);
signal reg_25_0, reg_25_1, reg_25_2, reg_25_3, reg_25_4, reg_25_5, reg_25_6, reg_25_7: std_logic_vector(4 downto 0);
signal reg_26_0, reg_26_1, reg_26_2, reg_26_3, reg_26_4, reg_26_5, reg_26_6, reg_26_7: std_logic_vector(4 downto 0);
signal reg_27_0, reg_27_1, reg_27_2, reg_27_3, reg_27_4, reg_27_5, reg_27_6, reg_27_7: std_logic_vector(4 downto 0);
signal reg_28_0, reg_28_1, reg_28_2, reg_28_3, reg_28_4, reg_28_5, reg_28_6, reg_28_7: std_logic_vector(4 downto 0);
signal reg_29_0, reg_29_1, reg_29_2, reg_29_3, reg_29_4, reg_29_5, reg_29_6, reg_29_7: std_logic_vector(4 downto 0);
signal reg_30_0, reg_30_1, reg_30_2, reg_30_3, reg_30_4, reg_30_5, reg_30_6, reg_30_7: std_logic_vector(4 downto 0);
signal reg_31_0, reg_31_1, reg_31_2, reg_31_3, reg_31_4, reg_31_5, reg_31_6, reg_31_7: std_logic_vector(4 downto 0);
signal reg_32_0, reg_32_1, reg_32_2, reg_32_3, reg_32_4, reg_32_5, reg_32_6, reg_32_7: std_logic_vector(4 downto 0);
signal reg_33_0, reg_33_1, reg_33_2, reg_33_3, reg_33_4, reg_33_5, reg_33_6, reg_33_7: std_logic_vector(4 downto 0);
signal reg_34_0, reg_34_1, reg_34_2, reg_34_3, reg_34_4, reg_34_5, reg_34_6, reg_34_7: std_logic_vector(4 downto 0);
signal reg_35_0, reg_35_1, reg_35_2, reg_35_3, reg_35_4, reg_35_5, reg_35_6, reg_35_7: std_logic_vector(4 downto 0);
signal reg_36_0, reg_36_1, reg_36_2, reg_36_3, reg_36_4, reg_36_5, reg_36_6, reg_36_7: std_logic_vector(4 downto 0);
signal reg_37_0, reg_37_1, reg_37_2, reg_37_3, reg_37_4, reg_37_5, reg_37_6, reg_37_7: std_logic_vector(4 downto 0);
signal reg_38_0, reg_38_1, reg_38_2, reg_38_3, reg_38_4, reg_38_5, reg_38_6, reg_38_7: std_logic_vector(4 downto 0);
signal reg_39_0, reg_39_1, reg_39_2, reg_39_3, reg_39_4, reg_39_5, reg_39_6, reg_39_7: std_logic_vector(4 downto 0);
signal reg_40_0, reg_40_1, reg_40_2, reg_40_3, reg_40_4, reg_40_5, reg_40_6, reg_40_7: std_logic_vector(4 downto 0);
signal reg_41_0, reg_41_1, reg_41_2, reg_41_3, reg_41_4, reg_41_5, reg_41_6, reg_41_7: std_logic_vector(4 downto 0);
signal reg_42_0, reg_42_1, reg_42_2, reg_42_3, reg_42_4, reg_42_5, reg_42_6, reg_42_7: std_logic_vector(4 downto 0);
signal reg_43_0, reg_43_1, reg_43_2, reg_43_3, reg_43_4, reg_43_5, reg_43_6, reg_43_7: std_logic_vector(4 downto 0);
signal reg_44_0, reg_44_1, reg_44_2, reg_44_3, reg_44_4, reg_44_5, reg_44_6, reg_44_7: std_logic_vector(4 downto 0);
signal reg_45_0, reg_45_1, reg_45_2, reg_45_3, reg_45_4, reg_45_5, reg_45_6, reg_45_7: std_logic_vector(4 downto 0);
signal reg_46_0, reg_46_1, reg_46_2, reg_46_3, reg_46_4, reg_46_5, reg_46_6, reg_46_7: std_logic_vector(4 downto 0);
signal reg_47_0, reg_47_1, reg_47_2, reg_47_3, reg_47_4, reg_47_5, reg_47_6, reg_47_7: std_logic_vector(4 downto 0);
signal reg_48_0, reg_48_1, reg_48_2, reg_48_3, reg_48_4, reg_48_5, reg_48_6, reg_48_7: std_logic_vector(4 downto 0);
signal reg_49_0, reg_49_1, reg_49_2, reg_49_3, reg_49_4, reg_49_5, reg_49_6, reg_49_7: std_logic_vector(4 downto 0);
signal reg_50_0, reg_50_1, reg_50_2, reg_50_3, reg_50_4, reg_50_5, reg_50_6, reg_50_7: std_logic_vector(4 downto 0);
signal reg_51_0, reg_51_1, reg_51_2, reg_51_3, reg_51_4, reg_51_5, reg_51_6, reg_51_7: std_logic_vector(4 downto 0);
signal reg_52_0, reg_52_1, reg_52_2, reg_52_3, reg_52_4, reg_52_5, reg_52_6, reg_52_7: std_logic_vector(4 downto 0);
signal reg_53_0, reg_53_1, reg_53_2, reg_53_3, reg_53_4, reg_53_5, reg_53_6, reg_53_7: std_logic_vector(4 downto 0);
signal reg_54_0, reg_54_1, reg_54_2, reg_54_3, reg_54_4, reg_54_5, reg_54_6, reg_54_7: std_logic_vector(4 downto 0);
signal reg_55_0, reg_55_1, reg_55_2, reg_55_3, reg_55_4, reg_55_5, reg_55_6, reg_55_7: std_logic_vector(4 downto 0);
signal reg_56_0, reg_56_1, reg_56_2, reg_56_3, reg_56_4, reg_56_5, reg_56_6, reg_56_7: std_logic_vector(4 downto 0);
signal reg_57_0, reg_57_1, reg_57_2, reg_57_3, reg_57_4, reg_57_5, reg_57_6, reg_57_7: std_logic_vector(4 downto 0);
signal reg_58_0, reg_58_1, reg_58_2, reg_58_3, reg_58_4, reg_58_5, reg_58_6, reg_58_7: std_logic_vector(4 downto 0);
signal reg_59_0, reg_59_1, reg_59_2, reg_59_3, reg_59_4, reg_59_5, reg_59_6, reg_59_7: std_logic_vector(4 downto 0);
signal reg_60_0, reg_60_1, reg_60_2, reg_60_3, reg_60_4, reg_60_5, reg_60_6, reg_60_7: std_logic_vector(4 downto 0);
signal reg_61_0, reg_61_1, reg_61_2, reg_61_3, reg_61_4, reg_61_5, reg_61_6, reg_61_7: std_logic_vector(4 downto 0);
signal reg_62_0, reg_62_1, reg_62_2, reg_62_3, reg_62_4, reg_62_5, reg_62_6, reg_62_7: std_logic_vector(4 downto 0);
signal reg_63_0, reg_63_1, reg_63_2, reg_63_3, reg_63_4, reg_63_5, reg_63_6, reg_63_7: std_logic_vector(4 downto 0);
signal enb_a, enb_b, enb_c, enb_d : std_logic;
signal enb_e, enb_f, enb_g, enb_h : std_logic;
signal CO9is1, CO256is0 : std_logic;
signal W8Data : std_logic_vector(1535 downto 0);
signal coeff_00, coeff_01, coeff_02, coeff_03, coeff_04, coeff_05, coeff_06, coeff_07, coeff_08, coeff_09: std_logic_vector(2 downto 0);
signal coeff_10, coeff_11, coeff_12, coeff_13, coeff_14, coeff_15, coeff_16, coeff_17, coeff_18, coeff_19: std_logic_vector(2 downto 0);
signal coeff_20, coeff_21, coeff_22, coeff_23, coeff_24, coeff_25, coeff_26, coeff_27, coeff_28, coeff_29: std_logic_vector(2 downto 0);
signal coeff_30, coeff_31, coeff_32, coeff_33, coeff_34, coeff_35, coeff_36, coeff_37, coeff_38, coeff_39: std_logic_vector(2 downto 0);
signal coeff_40, coeff_41, coeff_42, coeff_43, coeff_44, coeff_45, coeff_46, coeff_47, coeff_48, coeff_49: std_logic_vector(2 downto 0);
signal coeff_50, coeff_51, coeff_52, coeff_53, coeff_54, coeff_55, coeff_56, coeff_57, coeff_58, coeff_59: std_logic_vector(2 downto 0);
signal coeff_60, coeff_61, coeff_62, coeff_63: std_logic_vector(2 downto 0);

begin

CO9is1 <= '1' when CO9 = 1 else '0'; 
CO256is0 <= '1' when CO256 = 0 else '0';

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OutChSel_dly2 <= (others => '0');
		OutChSel_dly1 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		OutChSel_dly2 <= OutChSel_dly1;
		OutChSel_dly1 <= OutChSel;
	end if;  
end process;

W8Sel <= OutChSel_dly2(2 downto 0);

---------------------------------------------------------
  PixRomData <= 
X_RD_DATA( 1279 downto 1275 ) when CO256_dly2 = "00000000" else 
X_RD_DATA( 1274 downto 1270 ) when CO256_dly2 = "00000001" else 
X_RD_DATA( 1269 downto 1265 ) when CO256_dly2 = "00000010" else 
X_RD_DATA( 1264 downto 1260 ) when CO256_dly2 = "00000011" else 
X_RD_DATA( 1259 downto 1255 ) when CO256_dly2 = "00000100" else 
X_RD_DATA( 1254 downto 1250 ) when CO256_dly2 = "00000101" else 
X_RD_DATA( 1249 downto 1245 ) when CO256_dly2 = "00000110" else 
X_RD_DATA( 1244 downto 1240 ) when CO256_dly2 = "00000111" else 
X_RD_DATA( 1239 downto 1235 ) when CO256_dly2 = "00001000" else 
X_RD_DATA( 1234 downto 1230 ) when CO256_dly2 = "00001001" else 
X_RD_DATA( 1229 downto 1225 ) when CO256_dly2 = "00001010" else 
X_RD_DATA( 1224 downto 1220 ) when CO256_dly2 = "00001011" else 
X_RD_DATA( 1219 downto 1215 ) when CO256_dly2 = "00001100" else 
X_RD_DATA( 1214 downto 1210 ) when CO256_dly2 = "00001101" else 
X_RD_DATA( 1209 downto 1205 ) when CO256_dly2 = "00001110" else 
X_RD_DATA( 1204 downto 1200 ) when CO256_dly2 = "00001111" else 
X_RD_DATA( 1199 downto 1195 ) when CO256_dly2 = "00010000" else 
X_RD_DATA( 1194 downto 1190 ) when CO256_dly2 = "00010001" else 
X_RD_DATA( 1189 downto 1185 ) when CO256_dly2 = "00010010" else 
X_RD_DATA( 1184 downto 1180 ) when CO256_dly2 = "00010011" else 
X_RD_DATA( 1179 downto 1175 ) when CO256_dly2 = "00010100" else 
X_RD_DATA( 1174 downto 1170 ) when CO256_dly2 = "00010101" else 
X_RD_DATA( 1169 downto 1165 ) when CO256_dly2 = "00010110" else 
X_RD_DATA( 1164 downto 1160 ) when CO256_dly2 = "00010111" else 
X_RD_DATA( 1159 downto 1155 ) when CO256_dly2 = "00011000" else 
X_RD_DATA( 1154 downto 1150 ) when CO256_dly2 = "00011001" else 
X_RD_DATA( 1149 downto 1145 ) when CO256_dly2 = "00011010" else 
X_RD_DATA( 1144 downto 1140 ) when CO256_dly2 = "00011011" else 
X_RD_DATA( 1139 downto 1135 ) when CO256_dly2 = "00011100" else 
X_RD_DATA( 1134 downto 1130 ) when CO256_dly2 = "00011101" else 
X_RD_DATA( 1129 downto 1125 ) when CO256_dly2 = "00011110" else 
X_RD_DATA( 1124 downto 1120 ) when CO256_dly2 = "00011111" else 
X_RD_DATA( 1119 downto 1115 ) when CO256_dly2 = "00100000" else 
X_RD_DATA( 1114 downto 1110 ) when CO256_dly2 = "00100001" else 
X_RD_DATA( 1109 downto 1105 ) when CO256_dly2 = "00100010" else 
X_RD_DATA( 1104 downto 1100 ) when CO256_dly2 = "00100011" else 
X_RD_DATA( 1099 downto 1095 ) when CO256_dly2 = "00100100" else 
X_RD_DATA( 1094 downto 1090 ) when CO256_dly2 = "00100101" else 
X_RD_DATA( 1089 downto 1085 ) when CO256_dly2 = "00100110" else 
X_RD_DATA( 1084 downto 1080 ) when CO256_dly2 = "00100111" else 
X_RD_DATA( 1079 downto 1075 ) when CO256_dly2 = "00101000" else 
X_RD_DATA( 1074 downto 1070 ) when CO256_dly2 = "00101001" else 
X_RD_DATA( 1069 downto 1065 ) when CO256_dly2 = "00101010" else 
X_RD_DATA( 1064 downto 1060 ) when CO256_dly2 = "00101011" else 
X_RD_DATA( 1059 downto 1055 ) when CO256_dly2 = "00101100" else 
X_RD_DATA( 1054 downto 1050 ) when CO256_dly2 = "00101101" else 
X_RD_DATA( 1049 downto 1045 ) when CO256_dly2 = "00101110" else 
X_RD_DATA( 1044 downto 1040 ) when CO256_dly2 = "00101111" else 
X_RD_DATA( 1039 downto 1035 ) when CO256_dly2 = "00110000" else 
X_RD_DATA( 1034 downto 1030 ) when CO256_dly2 = "00110001" else 
X_RD_DATA( 1029 downto 1025 ) when CO256_dly2 = "00110010" else 
X_RD_DATA( 1024 downto 1020 ) when CO256_dly2 = "00110011" else 
X_RD_DATA( 1019 downto 1015 ) when CO256_dly2 = "00110100" else 
X_RD_DATA( 1014 downto 1010 ) when CO256_dly2 = "00110101" else 
X_RD_DATA( 1009 downto 1005 ) when CO256_dly2 = "00110110" else 
X_RD_DATA( 1004 downto 1000 ) when CO256_dly2 = "00110111" else 
X_RD_DATA( 999 downto 995 ) when CO256_dly2 = "00111000" else 
X_RD_DATA( 994 downto 990 ) when CO256_dly2 = "00111001" else 
X_RD_DATA( 989 downto 985 ) when CO256_dly2 = "00111010" else 
X_RD_DATA( 984 downto 980 ) when CO256_dly2 = "00111011" else 
X_RD_DATA( 979 downto 975 ) when CO256_dly2 = "00111100" else 
X_RD_DATA( 974 downto 970 ) when CO256_dly2 = "00111101" else 
X_RD_DATA( 969 downto 965 ) when CO256_dly2 = "00111110" else 
X_RD_DATA( 964 downto 960 ) when CO256_dly2 = "00111111" else 
X_RD_DATA( 959 downto 955 ) when CO256_dly2 = "01000000" else 
X_RD_DATA( 954 downto 950 ) when CO256_dly2 = "01000001" else 
X_RD_DATA( 949 downto 945 ) when CO256_dly2 = "01000010" else 
X_RD_DATA( 944 downto 940 ) when CO256_dly2 = "01000011" else 
X_RD_DATA( 939 downto 935 ) when CO256_dly2 = "01000100" else 
X_RD_DATA( 934 downto 930 ) when CO256_dly2 = "01000101" else 
X_RD_DATA( 929 downto 925 ) when CO256_dly2 = "01000110" else 
X_RD_DATA( 924 downto 920 ) when CO256_dly2 = "01000111" else 
X_RD_DATA( 919 downto 915 ) when CO256_dly2 = "01001000" else 
X_RD_DATA( 914 downto 910 ) when CO256_dly2 = "01001001" else 
X_RD_DATA( 909 downto 905 ) when CO256_dly2 = "01001010" else 
X_RD_DATA( 904 downto 900 ) when CO256_dly2 = "01001011" else 
X_RD_DATA( 899 downto 895 ) when CO256_dly2 = "01001100" else 
X_RD_DATA( 894 downto 890 ) when CO256_dly2 = "01001101" else 
X_RD_DATA( 889 downto 885 ) when CO256_dly2 = "01001110" else 
X_RD_DATA( 884 downto 880 ) when CO256_dly2 = "01001111" else 
X_RD_DATA( 879 downto 875 ) when CO256_dly2 = "01010000" else 
X_RD_DATA( 874 downto 870 ) when CO256_dly2 = "01010001" else 
X_RD_DATA( 869 downto 865 ) when CO256_dly2 = "01010010" else 
X_RD_DATA( 864 downto 860 ) when CO256_dly2 = "01010011" else 
X_RD_DATA( 859 downto 855 ) when CO256_dly2 = "01010100" else 
X_RD_DATA( 854 downto 850 ) when CO256_dly2 = "01010101" else 
X_RD_DATA( 849 downto 845 ) when CO256_dly2 = "01010110" else 
X_RD_DATA( 844 downto 840 ) when CO256_dly2 = "01010111" else 
X_RD_DATA( 839 downto 835 ) when CO256_dly2 = "01011000" else 
X_RD_DATA( 834 downto 830 ) when CO256_dly2 = "01011001" else 
X_RD_DATA( 829 downto 825 ) when CO256_dly2 = "01011010" else 
X_RD_DATA( 824 downto 820 ) when CO256_dly2 = "01011011" else 
X_RD_DATA( 819 downto 815 ) when CO256_dly2 = "01011100" else 
X_RD_DATA( 814 downto 810 ) when CO256_dly2 = "01011101" else 
X_RD_DATA( 809 downto 805 ) when CO256_dly2 = "01011110" else 
X_RD_DATA( 804 downto 800 ) when CO256_dly2 = "01011111" else 
X_RD_DATA( 799 downto 795 ) when CO256_dly2 = "01100000" else 
X_RD_DATA( 794 downto 790 ) when CO256_dly2 = "01100001" else 
X_RD_DATA( 789 downto 785 ) when CO256_dly2 = "01100010" else 
X_RD_DATA( 784 downto 780 ) when CO256_dly2 = "01100011" else 
X_RD_DATA( 779 downto 775 ) when CO256_dly2 = "01100100" else 
X_RD_DATA( 774 downto 770 ) when CO256_dly2 = "01100101" else 
X_RD_DATA( 769 downto 765 ) when CO256_dly2 = "01100110" else 
X_RD_DATA( 764 downto 760 ) when CO256_dly2 = "01100111" else 
X_RD_DATA( 759 downto 755 ) when CO256_dly2 = "01101000" else 
X_RD_DATA( 754 downto 750 ) when CO256_dly2 = "01101001" else 
X_RD_DATA( 749 downto 745 ) when CO256_dly2 = "01101010" else 
X_RD_DATA( 744 downto 740 ) when CO256_dly2 = "01101011" else 
X_RD_DATA( 739 downto 735 ) when CO256_dly2 = "01101100" else 
X_RD_DATA( 734 downto 730 ) when CO256_dly2 = "01101101" else 
X_RD_DATA( 729 downto 725 ) when CO256_dly2 = "01101110" else 
X_RD_DATA( 724 downto 720 ) when CO256_dly2 = "01101111" else 
X_RD_DATA( 719 downto 715 ) when CO256_dly2 = "01110000" else 
X_RD_DATA( 714 downto 710 ) when CO256_dly2 = "01110001" else 
X_RD_DATA( 709 downto 705 ) when CO256_dly2 = "01110010" else 
X_RD_DATA( 704 downto 700 ) when CO256_dly2 = "01110011" else 
X_RD_DATA( 699 downto 695 ) when CO256_dly2 = "01110100" else 
X_RD_DATA( 694 downto 690 ) when CO256_dly2 = "01110101" else 
X_RD_DATA( 689 downto 685 ) when CO256_dly2 = "01110110" else 
X_RD_DATA( 684 downto 680 ) when CO256_dly2 = "01110111" else 
X_RD_DATA( 679 downto 675 ) when CO256_dly2 = "01111000" else 
X_RD_DATA( 674 downto 670 ) when CO256_dly2 = "01111001" else 
X_RD_DATA( 669 downto 665 ) when CO256_dly2 = "01111010" else 
X_RD_DATA( 664 downto 660 ) when CO256_dly2 = "01111011" else 
X_RD_DATA( 659 downto 655 ) when CO256_dly2 = "01111100" else 
X_RD_DATA( 654 downto 650 ) when CO256_dly2 = "01111101" else 
X_RD_DATA( 649 downto 645 ) when CO256_dly2 = "01111110" else 
X_RD_DATA( 644 downto 640 ) when CO256_dly2 = "01111111" else 
X_RD_DATA( 639 downto 635 ) when CO256_dly2 = "10000000" else 
X_RD_DATA( 634 downto 630 ) when CO256_dly2 = "10000001" else 
X_RD_DATA( 629 downto 625 ) when CO256_dly2 = "10000010" else 
X_RD_DATA( 624 downto 620 ) when CO256_dly2 = "10000011" else 
X_RD_DATA( 619 downto 615 ) when CO256_dly2 = "10000100" else 
X_RD_DATA( 614 downto 610 ) when CO256_dly2 = "10000101" else 
X_RD_DATA( 609 downto 605 ) when CO256_dly2 = "10000110" else 
X_RD_DATA( 604 downto 600 ) when CO256_dly2 = "10000111" else 
X_RD_DATA( 599 downto 595 ) when CO256_dly2 = "10001000" else 
X_RD_DATA( 594 downto 590 ) when CO256_dly2 = "10001001" else 
X_RD_DATA( 589 downto 585 ) when CO256_dly2 = "10001010" else 
X_RD_DATA( 584 downto 580 ) when CO256_dly2 = "10001011" else 
X_RD_DATA( 579 downto 575 ) when CO256_dly2 = "10001100" else 
X_RD_DATA( 574 downto 570 ) when CO256_dly2 = "10001101" else 
X_RD_DATA( 569 downto 565 ) when CO256_dly2 = "10001110" else 
X_RD_DATA( 564 downto 560 ) when CO256_dly2 = "10001111" else 
X_RD_DATA( 559 downto 555 ) when CO256_dly2 = "10010000" else 
X_RD_DATA( 554 downto 550 ) when CO256_dly2 = "10010001" else 
X_RD_DATA( 549 downto 545 ) when CO256_dly2 = "10010010" else 
X_RD_DATA( 544 downto 540 ) when CO256_dly2 = "10010011" else 
X_RD_DATA( 539 downto 535 ) when CO256_dly2 = "10010100" else 
X_RD_DATA( 534 downto 530 ) when CO256_dly2 = "10010101" else 
X_RD_DATA( 529 downto 525 ) when CO256_dly2 = "10010110" else 
X_RD_DATA( 524 downto 520 ) when CO256_dly2 = "10010111" else 
X_RD_DATA( 519 downto 515 ) when CO256_dly2 = "10011000" else 
X_RD_DATA( 514 downto 510 ) when CO256_dly2 = "10011001" else 
X_RD_DATA( 509 downto 505 ) when CO256_dly2 = "10011010" else 
X_RD_DATA( 504 downto 500 ) when CO256_dly2 = "10011011" else 
X_RD_DATA( 499 downto 495 ) when CO256_dly2 = "10011100" else 
X_RD_DATA( 494 downto 490 ) when CO256_dly2 = "10011101" else 
X_RD_DATA( 489 downto 485 ) when CO256_dly2 = "10011110" else 
X_RD_DATA( 484 downto 480 ) when CO256_dly2 = "10011111" else 
X_RD_DATA( 479 downto 475 ) when CO256_dly2 = "10100000" else 
X_RD_DATA( 474 downto 470 ) when CO256_dly2 = "10100001" else 
X_RD_DATA( 469 downto 465 ) when CO256_dly2 = "10100010" else 
X_RD_DATA( 464 downto 460 ) when CO256_dly2 = "10100011" else 
X_RD_DATA( 459 downto 455 ) when CO256_dly2 = "10100100" else 
X_RD_DATA( 454 downto 450 ) when CO256_dly2 = "10100101" else 
X_RD_DATA( 449 downto 445 ) when CO256_dly2 = "10100110" else 
X_RD_DATA( 444 downto 440 ) when CO256_dly2 = "10100111" else 
X_RD_DATA( 439 downto 435 ) when CO256_dly2 = "10101000" else 
X_RD_DATA( 434 downto 430 ) when CO256_dly2 = "10101001" else 
X_RD_DATA( 429 downto 425 ) when CO256_dly2 = "10101010" else 
X_RD_DATA( 424 downto 420 ) when CO256_dly2 = "10101011" else 
X_RD_DATA( 419 downto 415 ) when CO256_dly2 = "10101100" else 
X_RD_DATA( 414 downto 410 ) when CO256_dly2 = "10101101" else 
X_RD_DATA( 409 downto 405 ) when CO256_dly2 = "10101110" else 
X_RD_DATA( 404 downto 400 ) when CO256_dly2 = "10101111" else 
X_RD_DATA( 399 downto 395 ) when CO256_dly2 = "10110000" else 
X_RD_DATA( 394 downto 390 ) when CO256_dly2 = "10110001" else 
X_RD_DATA( 389 downto 385 ) when CO256_dly2 = "10110010" else 
X_RD_DATA( 384 downto 380 ) when CO256_dly2 = "10110011" else 
X_RD_DATA( 379 downto 375 ) when CO256_dly2 = "10110100" else 
X_RD_DATA( 374 downto 370 ) when CO256_dly2 = "10110101" else 
X_RD_DATA( 369 downto 365 ) when CO256_dly2 = "10110110" else 
X_RD_DATA( 364 downto 360 ) when CO256_dly2 = "10110111" else 
X_RD_DATA( 359 downto 355 ) when CO256_dly2 = "10111000" else 
X_RD_DATA( 354 downto 350 ) when CO256_dly2 = "10111001" else 
X_RD_DATA( 349 downto 345 ) when CO256_dly2 = "10111010" else 
X_RD_DATA( 344 downto 340 ) when CO256_dly2 = "10111011" else 
X_RD_DATA( 339 downto 335 ) when CO256_dly2 = "10111100" else 
X_RD_DATA( 334 downto 330 ) when CO256_dly2 = "10111101" else 
X_RD_DATA( 329 downto 325 ) when CO256_dly2 = "10111110" else 
X_RD_DATA( 324 downto 320 ) when CO256_dly2 = "10111111" else 
X_RD_DATA( 319 downto 315 ) when CO256_dly2 = "11000000" else 
X_RD_DATA( 314 downto 310 ) when CO256_dly2 = "11000001" else 
X_RD_DATA( 309 downto 305 ) when CO256_dly2 = "11000010" else 
X_RD_DATA( 304 downto 300 ) when CO256_dly2 = "11000011" else 
X_RD_DATA( 299 downto 295 ) when CO256_dly2 = "11000100" else 
X_RD_DATA( 294 downto 290 ) when CO256_dly2 = "11000101" else 
X_RD_DATA( 289 downto 285 ) when CO256_dly2 = "11000110" else 
X_RD_DATA( 284 downto 280 ) when CO256_dly2 = "11000111" else 
X_RD_DATA( 279 downto 275 ) when CO256_dly2 = "11001000" else 
X_RD_DATA( 274 downto 270 ) when CO256_dly2 = "11001001" else 
X_RD_DATA( 269 downto 265 ) when CO256_dly2 = "11001010" else 
X_RD_DATA( 264 downto 260 ) when CO256_dly2 = "11001011" else 
X_RD_DATA( 259 downto 255 ) when CO256_dly2 = "11001100" else 
X_RD_DATA( 254 downto 250 ) when CO256_dly2 = "11001101" else 
X_RD_DATA( 249 downto 245 ) when CO256_dly2 = "11001110" else 
X_RD_DATA( 244 downto 240 ) when CO256_dly2 = "11001111" else 
X_RD_DATA( 239 downto 235 ) when CO256_dly2 = "11010000" else 
X_RD_DATA( 234 downto 230 ) when CO256_dly2 = "11010001" else 
X_RD_DATA( 229 downto 225 ) when CO256_dly2 = "11010010" else 
X_RD_DATA( 224 downto 220 ) when CO256_dly2 = "11010011" else 
X_RD_DATA( 219 downto 215 ) when CO256_dly2 = "11010100" else 
X_RD_DATA( 214 downto 210 ) when CO256_dly2 = "11010101" else 
X_RD_DATA( 209 downto 205 ) when CO256_dly2 = "11010110" else 
X_RD_DATA( 204 downto 200 ) when CO256_dly2 = "11010111" else 
X_RD_DATA( 199 downto 195 ) when CO256_dly2 = "11011000" else 
X_RD_DATA( 194 downto 190 ) when CO256_dly2 = "11011001" else 
X_RD_DATA( 189 downto 185 ) when CO256_dly2 = "11011010" else 
X_RD_DATA( 184 downto 180 ) when CO256_dly2 = "11011011" else 
X_RD_DATA( 179 downto 175 ) when CO256_dly2 = "11011100" else 
X_RD_DATA( 174 downto 170 ) when CO256_dly2 = "11011101" else 
X_RD_DATA( 169 downto 165 ) when CO256_dly2 = "11011110" else 
X_RD_DATA( 164 downto 160 ) when CO256_dly2 = "11011111" else 
X_RD_DATA( 159 downto 155 ) when CO256_dly2 = "11100000" else 
X_RD_DATA( 154 downto 150 ) when CO256_dly2 = "11100001" else 
X_RD_DATA( 149 downto 145 ) when CO256_dly2 = "11100010" else 
X_RD_DATA( 144 downto 140 ) when CO256_dly2 = "11100011" else 
X_RD_DATA( 139 downto 135 ) when CO256_dly2 = "11100100" else 
X_RD_DATA( 134 downto 130 ) when CO256_dly2 = "11100101" else 
X_RD_DATA( 129 downto 125 ) when CO256_dly2 = "11100110" else 
X_RD_DATA( 124 downto 120 ) when CO256_dly2 = "11100111" else 
X_RD_DATA( 119 downto 115 ) when CO256_dly2 = "11101000" else 
X_RD_DATA( 114 downto 110 ) when CO256_dly2 = "11101001" else 
X_RD_DATA( 109 downto 105 ) when CO256_dly2 = "11101010" else 
X_RD_DATA( 104 downto 100 ) when CO256_dly2 = "11101011" else 
X_RD_DATA( 99 downto 95 ) when CO256_dly2 = "11101100" else 
X_RD_DATA( 94 downto 90 ) when CO256_dly2 = "11101101" else 
X_RD_DATA( 89 downto 85 ) when CO256_dly2 = "11101110" else 
X_RD_DATA( 84 downto 80 ) when CO256_dly2 = "11101111" else 
X_RD_DATA( 79 downto 75 ) when CO256_dly2 = "11110000" else 
X_RD_DATA( 74 downto 70 ) when CO256_dly2 = "11110001" else 
X_RD_DATA( 69 downto 65 ) when CO256_dly2 = "11110010" else 
X_RD_DATA( 64 downto 60 ) when CO256_dly2 = "11110011" else 
X_RD_DATA( 59 downto 55 ) when CO256_dly2 = "11110100" else 
X_RD_DATA( 54 downto 50 ) when CO256_dly2 = "11110101" else 
X_RD_DATA( 49 downto 45 ) when CO256_dly2 = "11110110" else 
X_RD_DATA( 44 downto 40 ) when CO256_dly2 = "11110111" else 
X_RD_DATA( 39 downto 35 ) when CO256_dly2 = "11111000" else 
X_RD_DATA( 34 downto 30 ) when CO256_dly2 = "11111001" else 
X_RD_DATA( 29 downto 25 ) when CO256_dly2 = "11111010" else 
X_RD_DATA( 24 downto 20 ) when CO256_dly2 = "11111011" else 
X_RD_DATA( 19 downto 15 ) when CO256_dly2 = "11111100" else 
X_RD_DATA( 14 downto 10 ) when CO256_dly2 = "11111101" else 
X_RD_DATA( 9 downto 5 ) when CO256_dly2 = "11111110" else 
X_RD_DATA( 4 downto 0 ) ; -- when CO256_dly2 = "11111111" else 


W8Addr_L <= CO9;
W8Addr_H <= ("0" & CO256 & "000") + ("0000" & CO256); -- CO256 x 9
W8Addr_prev <= W8Addr_H + ("0000000" & W8Addr_L);

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		W8Addr <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if(W8Addr_prev >= -448 and W8Addr_prev <= -1) then -- note that this is Signed
			W8Addr <= (others => '0');
		else
			W8Addr <= W8Addr_prev;
		end if;
	end if;  
end process;

---------------------------------------------------------
enb_a <= '1' when W8Sel = 1 and CO256 = 255 and CO9 = 8 else '0';
enb_b <= '1' when W8Sel = 2 and CO256 = 255 and CO9 = 8 else '0';
enb_c <= '1' when W8Sel = 3 and CO256 = 255 and CO9 = 8 else '0';
enb_d <= '1' when W8Sel = 4 and CO256 = 255 and CO9 = 8 else '0';
enb_e <= '1' when W8Sel = 5 and CO256 = 255 and CO9 = 8 else '0';
enb_f <= '1' when W8Sel = 6 and CO256 = 255 and CO9 = 8 else '0';
enb_g <= '1' when W8Sel = 7 and CO256 = 255 and CO9 = 8 else '0';
enb_h <= '1' when W8Sel = 0 and CO256 = 255 and CO9 = 8 else '0';

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			outCh_000 <= reg_00_0;
			outCh_001 <= reg_00_1;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_00_0 <= Relu_00;
		elsif(enb_b = '1') then
			reg_00_1 <= Relu_00;
		elsif(enb_c = '1') then
			reg_00_2 <= Relu_00;
		elsif(enb_d = '1') then
			reg_00_3 <= Relu_00;
		elsif(enb_e = '1') then
			reg_00_4 <= Relu_00;
		elsif(enb_f = '1') then
			reg_00_5 <= Relu_00;
		elsif(enb_g = '1') then
			reg_00_6 <= Relu_00;
		elsif(enb_h = '1') then
			reg_00_7 <= Relu_00;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_01_0 <= Relu_01;
		elsif(enb_b = '1') then
			reg_01_1 <= Relu_01;
		elsif(enb_c = '1') then
			reg_01_2 <= Relu_01;
		elsif(enb_d = '1') then
			reg_01_3 <= Relu_01;
		elsif(enb_e = '1') then
			reg_01_4 <= Relu_01;
		elsif(enb_f = '1') then
			reg_01_5 <= Relu_01;
		elsif(enb_g = '1') then
			reg_01_6 <= Relu_01;
		elsif(enb_h = '1') then
			reg_01_7 <= Relu_01;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_02_0 <= Relu_02;
		elsif(enb_b = '1') then
			reg_02_1 <= Relu_02;
		elsif(enb_c = '1') then
			reg_02_2 <= Relu_02;
		elsif(enb_d = '1') then
			reg_02_3 <= Relu_02;
		elsif(enb_e = '1') then
			reg_02_4 <= Relu_02;
		elsif(enb_f = '1') then
			reg_02_5 <= Relu_02;
		elsif(enb_g = '1') then
			reg_02_6 <= Relu_02;
		elsif(enb_h = '1') then
			reg_02_7 <= Relu_02;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_03_0 <= Relu_03;
		elsif(enb_b = '1') then
			reg_03_1 <= Relu_03;
		elsif(enb_c = '1') then
			reg_03_2 <= Relu_03;
		elsif(enb_d = '1') then
			reg_03_3 <= Relu_03;
		elsif(enb_e = '1') then
			reg_03_4 <= Relu_03;
		elsif(enb_f = '1') then
			reg_03_5 <= Relu_03;
		elsif(enb_g = '1') then
			reg_03_6 <= Relu_03;
		elsif(enb_h = '1') then
			reg_03_7 <= Relu_03;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_04_0 <= Relu_04;
		elsif(enb_b = '1') then
			reg_04_1 <= Relu_04;
		elsif(enb_c = '1') then
			reg_04_2 <= Relu_04;
		elsif(enb_d = '1') then
			reg_04_3 <= Relu_04;
		elsif(enb_e = '1') then
			reg_04_4 <= Relu_04;
		elsif(enb_f = '1') then
			reg_04_5 <= Relu_04;
		elsif(enb_g = '1') then
			reg_04_6 <= Relu_04;
		elsif(enb_h = '1') then
			reg_04_7 <= Relu_04;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_05_0 <= Relu_05;
		elsif(enb_b = '1') then
			reg_05_1 <= Relu_05;
		elsif(enb_c = '1') then
			reg_05_2 <= Relu_05;
		elsif(enb_d = '1') then
			reg_05_3 <= Relu_05;
		elsif(enb_e = '1') then
			reg_05_4 <= Relu_05;
		elsif(enb_f = '1') then
			reg_05_5 <= Relu_05;
		elsif(enb_g = '1') then
			reg_05_6 <= Relu_05;
		elsif(enb_h = '1') then
			reg_05_7 <= Relu_05;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_06_0 <= Relu_06;
		elsif(enb_b = '1') then
			reg_06_1 <= Relu_06;
		elsif(enb_c = '1') then
			reg_06_2 <= Relu_06;
		elsif(enb_d = '1') then
			reg_06_3 <= Relu_06;
		elsif(enb_e = '1') then
			reg_06_4 <= Relu_06;
		elsif(enb_f = '1') then
			reg_06_5 <= Relu_06;
		elsif(enb_g = '1') then
			reg_06_6 <= Relu_06;
		elsif(enb_h = '1') then
			reg_06_7 <= Relu_06;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_07_0 <= Relu_07;
		elsif(enb_b = '1') then
			reg_07_1 <= Relu_07;
		elsif(enb_c = '1') then
			reg_07_2 <= Relu_07;
		elsif(enb_d = '1') then
			reg_07_3 <= Relu_07;
		elsif(enb_e = '1') then
			reg_07_4 <= Relu_07;
		elsif(enb_f = '1') then
			reg_07_5 <= Relu_07;
		elsif(enb_g = '1') then
			reg_07_6 <= Relu_07;
		elsif(enb_h = '1') then
			reg_07_7 <= Relu_07;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_08_0 <= Relu_08;
		elsif(enb_b = '1') then
			reg_08_1 <= Relu_08;
		elsif(enb_c = '1') then
			reg_08_2 <= Relu_08;
		elsif(enb_d = '1') then
			reg_08_3 <= Relu_08;
		elsif(enb_e = '1') then
			reg_08_4 <= Relu_08;
		elsif(enb_f = '1') then
			reg_08_5 <= Relu_08;
		elsif(enb_g = '1') then
			reg_08_6 <= Relu_08;
		elsif(enb_h = '1') then
			reg_08_7 <= Relu_08;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_09_0 <= Relu_09;
		elsif(enb_b = '1') then
			reg_09_1 <= Relu_09;
		elsif(enb_c = '1') then
			reg_09_2 <= Relu_09;
		elsif(enb_d = '1') then
			reg_09_3 <= Relu_09;
		elsif(enb_e = '1') then
			reg_09_4 <= Relu_09;
		elsif(enb_f = '1') then
			reg_09_5 <= Relu_09;
		elsif(enb_g = '1') then
			reg_09_6 <= Relu_09;
		elsif(enb_h = '1') then
			reg_09_7 <= Relu_09;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_10_0 <= Relu_10;
		elsif(enb_b = '1') then
			reg_10_1 <= Relu_10;
		elsif(enb_c = '1') then
			reg_10_2 <= Relu_10;
		elsif(enb_d = '1') then
			reg_10_3 <= Relu_10;
		elsif(enb_e = '1') then
			reg_10_4 <= Relu_10;
		elsif(enb_f = '1') then
			reg_10_5 <= Relu_10;
		elsif(enb_g = '1') then
			reg_10_6 <= Relu_10;
		elsif(enb_h = '1') then
			reg_10_7 <= Relu_10;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_11_0 <= Relu_11;
		elsif(enb_b = '1') then
			reg_11_1 <= Relu_11;
		elsif(enb_c = '1') then
			reg_11_2 <= Relu_11;
		elsif(enb_d = '1') then
			reg_11_3 <= Relu_11;
		elsif(enb_e = '1') then
			reg_11_4 <= Relu_11;
		elsif(enb_f = '1') then
			reg_11_5 <= Relu_11;
		elsif(enb_g = '1') then
			reg_11_6 <= Relu_11;
		elsif(enb_h = '1') then
			reg_11_7 <= Relu_11;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_12_0 <= Relu_12;
		elsif(enb_b = '1') then
			reg_12_1 <= Relu_12;
		elsif(enb_c = '1') then
			reg_12_2 <= Relu_12;
		elsif(enb_d = '1') then
			reg_12_3 <= Relu_12;
		elsif(enb_e = '1') then
			reg_12_4 <= Relu_12;
		elsif(enb_f = '1') then
			reg_12_5 <= Relu_12;
		elsif(enb_g = '1') then
			reg_12_6 <= Relu_12;
		elsif(enb_h = '1') then
			reg_12_7 <= Relu_12;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_13_0 <= Relu_13;
		elsif(enb_b = '1') then
			reg_13_1 <= Relu_13;
		elsif(enb_c = '1') then
			reg_13_2 <= Relu_13;
		elsif(enb_d = '1') then
			reg_13_3 <= Relu_13;
		elsif(enb_e = '1') then
			reg_13_4 <= Relu_13;
		elsif(enb_f = '1') then
			reg_13_5 <= Relu_13;
		elsif(enb_g = '1') then
			reg_13_6 <= Relu_13;
		elsif(enb_h = '1') then
			reg_13_7 <= Relu_13;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_14_0 <= Relu_14;
		elsif(enb_b = '1') then
			reg_14_1 <= Relu_14;
		elsif(enb_c = '1') then
			reg_14_2 <= Relu_14;
		elsif(enb_d = '1') then
			reg_14_3 <= Relu_14;
		elsif(enb_e = '1') then
			reg_14_4 <= Relu_14;
		elsif(enb_f = '1') then
			reg_14_5 <= Relu_14;
		elsif(enb_g = '1') then
			reg_14_6 <= Relu_14;
		elsif(enb_h = '1') then
			reg_14_7 <= Relu_14;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_15_0 <= Relu_15;
		elsif(enb_b = '1') then
			reg_15_1 <= Relu_15;
		elsif(enb_c = '1') then
			reg_15_2 <= Relu_15;
		elsif(enb_d = '1') then
			reg_15_3 <= Relu_15;
		elsif(enb_e = '1') then
			reg_15_4 <= Relu_15;
		elsif(enb_f = '1') then
			reg_15_5 <= Relu_15;
		elsif(enb_g = '1') then
			reg_15_6 <= Relu_15;
		elsif(enb_h = '1') then
			reg_15_7 <= Relu_15;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_16_0 <= Relu_16;
		elsif(enb_b = '1') then
			reg_16_1 <= Relu_16;
		elsif(enb_c = '1') then
			reg_16_2 <= Relu_16;
		elsif(enb_d = '1') then
			reg_16_3 <= Relu_16;
		elsif(enb_e = '1') then
			reg_16_4 <= Relu_16;
		elsif(enb_f = '1') then
			reg_16_5 <= Relu_16;
		elsif(enb_g = '1') then
			reg_16_6 <= Relu_16;
		elsif(enb_h = '1') then
			reg_16_7 <= Relu_16;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_17_0 <= Relu_17;
		elsif(enb_b = '1') then
			reg_17_1 <= Relu_17;
		elsif(enb_c = '1') then
			reg_17_2 <= Relu_17;
		elsif(enb_d = '1') then
			reg_17_3 <= Relu_17;
		elsif(enb_e = '1') then
			reg_17_4 <= Relu_17;
		elsif(enb_f = '1') then
			reg_17_5 <= Relu_17;
		elsif(enb_g = '1') then
			reg_17_6 <= Relu_17;
		elsif(enb_h = '1') then
			reg_17_7 <= Relu_17;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_18_0 <= Relu_18;
		elsif(enb_b = '1') then
			reg_18_1 <= Relu_18;
		elsif(enb_c = '1') then
			reg_18_2 <= Relu_18;
		elsif(enb_d = '1') then
			reg_18_3 <= Relu_18;
		elsif(enb_e = '1') then
			reg_18_4 <= Relu_18;
		elsif(enb_f = '1') then
			reg_18_5 <= Relu_18;
		elsif(enb_g = '1') then
			reg_18_6 <= Relu_18;
		elsif(enb_h = '1') then
			reg_18_7 <= Relu_18;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_19_0 <= Relu_19;
		elsif(enb_b = '1') then
			reg_19_1 <= Relu_19;
		elsif(enb_c = '1') then
			reg_19_2 <= Relu_19;
		elsif(enb_d = '1') then
			reg_19_3 <= Relu_19;
		elsif(enb_e = '1') then
			reg_19_4 <= Relu_19;
		elsif(enb_f = '1') then
			reg_19_5 <= Relu_19;
		elsif(enb_g = '1') then
			reg_19_6 <= Relu_19;
		elsif(enb_h = '1') then
			reg_19_7 <= Relu_19;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_20_0 <= Relu_20;
		elsif(enb_b = '1') then
			reg_20_1 <= Relu_20;
		elsif(enb_c = '1') then
			reg_20_2 <= Relu_20;
		elsif(enb_d = '1') then
			reg_20_3 <= Relu_20;
		elsif(enb_e = '1') then
			reg_20_4 <= Relu_20;
		elsif(enb_f = '1') then
			reg_20_5 <= Relu_20;
		elsif(enb_g = '1') then
			reg_20_6 <= Relu_20;
		elsif(enb_h = '1') then
			reg_20_7 <= Relu_20;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_21_0 <= Relu_21;
		elsif(enb_b = '1') then
			reg_21_1 <= Relu_21;
		elsif(enb_c = '1') then
			reg_21_2 <= Relu_21;
		elsif(enb_d = '1') then
			reg_21_3 <= Relu_21;
		elsif(enb_e = '1') then
			reg_21_4 <= Relu_21;
		elsif(enb_f = '1') then
			reg_21_5 <= Relu_21;
		elsif(enb_g = '1') then
			reg_21_6 <= Relu_21;
		elsif(enb_h = '1') then
			reg_21_7 <= Relu_21;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_22_0 <= Relu_22;
		elsif(enb_b = '1') then
			reg_22_1 <= Relu_22;
		elsif(enb_c = '1') then
			reg_22_2 <= Relu_22;
		elsif(enb_d = '1') then
			reg_22_3 <= Relu_22;
		elsif(enb_e = '1') then
			reg_22_4 <= Relu_22;
		elsif(enb_f = '1') then
			reg_22_5 <= Relu_22;
		elsif(enb_g = '1') then
			reg_22_6 <= Relu_22;
		elsif(enb_h = '1') then
			reg_22_7 <= Relu_22;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_23_0 <= Relu_23;
		elsif(enb_b = '1') then
			reg_23_1 <= Relu_23;
		elsif(enb_c = '1') then
			reg_23_2 <= Relu_23;
		elsif(enb_d = '1') then
			reg_23_3 <= Relu_23;
		elsif(enb_e = '1') then
			reg_23_4 <= Relu_23;
		elsif(enb_f = '1') then
			reg_23_5 <= Relu_23;
		elsif(enb_g = '1') then
			reg_23_6 <= Relu_23;
		elsif(enb_h = '1') then
			reg_23_7 <= Relu_23;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_24_0 <= Relu_24;
		elsif(enb_b = '1') then
			reg_24_1 <= Relu_24;
		elsif(enb_c = '1') then
			reg_24_2 <= Relu_24;
		elsif(enb_d = '1') then
			reg_24_3 <= Relu_24;
		elsif(enb_e = '1') then
			reg_24_4 <= Relu_24;
		elsif(enb_f = '1') then
			reg_24_5 <= Relu_24;
		elsif(enb_g = '1') then
			reg_24_6 <= Relu_24;
		elsif(enb_h = '1') then
			reg_24_7 <= Relu_24;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_25_0 <= Relu_25;
		elsif(enb_b = '1') then
			reg_25_1 <= Relu_25;
		elsif(enb_c = '1') then
			reg_25_2 <= Relu_25;
		elsif(enb_d = '1') then
			reg_25_3 <= Relu_25;
		elsif(enb_e = '1') then
			reg_25_4 <= Relu_25;
		elsif(enb_f = '1') then
			reg_25_5 <= Relu_25;
		elsif(enb_g = '1') then
			reg_25_6 <= Relu_25;
		elsif(enb_h = '1') then
			reg_25_7 <= Relu_25;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_26_0 <= Relu_26;
		elsif(enb_b = '1') then
			reg_26_1 <= Relu_26;
		elsif(enb_c = '1') then
			reg_26_2 <= Relu_26;
		elsif(enb_d = '1') then
			reg_26_3 <= Relu_26;
		elsif(enb_e = '1') then
			reg_26_4 <= Relu_26;
		elsif(enb_f = '1') then
			reg_26_5 <= Relu_26;
		elsif(enb_g = '1') then
			reg_26_6 <= Relu_26;
		elsif(enb_h = '1') then
			reg_26_7 <= Relu_26;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_27_0 <= Relu_27;
		elsif(enb_b = '1') then
			reg_27_1 <= Relu_27;
		elsif(enb_c = '1') then
			reg_27_2 <= Relu_27;
		elsif(enb_d = '1') then
			reg_27_3 <= Relu_27;
		elsif(enb_e = '1') then
			reg_27_4 <= Relu_27;
		elsif(enb_f = '1') then
			reg_27_5 <= Relu_27;
		elsif(enb_g = '1') then
			reg_27_6 <= Relu_27;
		elsif(enb_h = '1') then
			reg_27_7 <= Relu_27;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_28_0 <= Relu_28;
		elsif(enb_b = '1') then
			reg_28_1 <= Relu_28;
		elsif(enb_c = '1') then
			reg_28_2 <= Relu_28;
		elsif(enb_d = '1') then
			reg_28_3 <= Relu_28;
		elsif(enb_e = '1') then
			reg_28_4 <= Relu_28;
		elsif(enb_f = '1') then
			reg_28_5 <= Relu_28;
		elsif(enb_g = '1') then
			reg_28_6 <= Relu_28;
		elsif(enb_h = '1') then
			reg_28_7 <= Relu_28;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_29_0 <= Relu_29;
		elsif(enb_b = '1') then
			reg_29_1 <= Relu_29;
		elsif(enb_c = '1') then
			reg_29_2 <= Relu_29;
		elsif(enb_d = '1') then
			reg_29_3 <= Relu_29;
		elsif(enb_e = '1') then
			reg_29_4 <= Relu_29;
		elsif(enb_f = '1') then
			reg_29_5 <= Relu_29;
		elsif(enb_g = '1') then
			reg_29_6 <= Relu_29;
		elsif(enb_h = '1') then
			reg_29_7 <= Relu_29;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_30_0 <= Relu_30;
		elsif(enb_b = '1') then
			reg_30_1 <= Relu_30;
		elsif(enb_c = '1') then
			reg_30_2 <= Relu_30;
		elsif(enb_d = '1') then
			reg_30_3 <= Relu_30;
		elsif(enb_e = '1') then
			reg_30_4 <= Relu_30;
		elsif(enb_f = '1') then
			reg_30_5 <= Relu_30;
		elsif(enb_g = '1') then
			reg_30_6 <= Relu_30;
		elsif(enb_h = '1') then
			reg_30_7 <= Relu_30;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_31_0 <= Relu_31;
		elsif(enb_b = '1') then
			reg_31_1 <= Relu_31;
		elsif(enb_c = '1') then
			reg_31_2 <= Relu_31;
		elsif(enb_d = '1') then
			reg_31_3 <= Relu_31;
		elsif(enb_e = '1') then
			reg_31_4 <= Relu_31;
		elsif(enb_f = '1') then
			reg_31_5 <= Relu_31;
		elsif(enb_g = '1') then
			reg_31_6 <= Relu_31;
		elsif(enb_h = '1') then
			reg_31_7 <= Relu_31;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_32_0 <= Relu_32;
		elsif(enb_b = '1') then
			reg_32_1 <= Relu_32;
		elsif(enb_c = '1') then
			reg_32_2 <= Relu_32;
		elsif(enb_d = '1') then
			reg_32_3 <= Relu_32;
		elsif(enb_e = '1') then
			reg_32_4 <= Relu_32;
		elsif(enb_f = '1') then
			reg_32_5 <= Relu_32;
		elsif(enb_g = '1') then
			reg_32_6 <= Relu_32;
		elsif(enb_h = '1') then
			reg_32_7 <= Relu_32;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_33_0 <= Relu_33;
		elsif(enb_b = '1') then
			reg_33_1 <= Relu_33;
		elsif(enb_c = '1') then
			reg_33_2 <= Relu_33;
		elsif(enb_d = '1') then
			reg_33_3 <= Relu_33;
		elsif(enb_e = '1') then
			reg_33_4 <= Relu_33;
		elsif(enb_f = '1') then
			reg_33_5 <= Relu_33;
		elsif(enb_g = '1') then
			reg_33_6 <= Relu_33;
		elsif(enb_h = '1') then
			reg_33_7 <= Relu_33;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_34_0 <= Relu_34;
		elsif(enb_b = '1') then
			reg_34_1 <= Relu_34;
		elsif(enb_c = '1') then
			reg_34_2 <= Relu_34;
		elsif(enb_d = '1') then
			reg_34_3 <= Relu_34;
		elsif(enb_e = '1') then
			reg_34_4 <= Relu_34;
		elsif(enb_f = '1') then
			reg_34_5 <= Relu_34;
		elsif(enb_g = '1') then
			reg_34_6 <= Relu_34;
		elsif(enb_h = '1') then
			reg_34_7 <= Relu_34;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_35_0 <= Relu_35;
		elsif(enb_b = '1') then
			reg_35_1 <= Relu_35;
		elsif(enb_c = '1') then
			reg_35_2 <= Relu_35;
		elsif(enb_d = '1') then
			reg_35_3 <= Relu_35;
		elsif(enb_e = '1') then
			reg_35_4 <= Relu_35;
		elsif(enb_f = '1') then
			reg_35_5 <= Relu_35;
		elsif(enb_g = '1') then
			reg_35_6 <= Relu_35;
		elsif(enb_h = '1') then
			reg_35_7 <= Relu_35;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_36_0 <= Relu_36;
		elsif(enb_b = '1') then
			reg_36_1 <= Relu_36;
		elsif(enb_c = '1') then
			reg_36_2 <= Relu_36;
		elsif(enb_d = '1') then
			reg_36_3 <= Relu_36;
		elsif(enb_e = '1') then
			reg_36_4 <= Relu_36;
		elsif(enb_f = '1') then
			reg_36_5 <= Relu_36;
		elsif(enb_g = '1') then
			reg_36_6 <= Relu_36;
		elsif(enb_h = '1') then
			reg_36_7 <= Relu_36;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_37_0 <= Relu_37;
		elsif(enb_b = '1') then
			reg_37_1 <= Relu_37;
		elsif(enb_c = '1') then
			reg_37_2 <= Relu_37;
		elsif(enb_d = '1') then
			reg_37_3 <= Relu_37;
		elsif(enb_e = '1') then
			reg_37_4 <= Relu_37;
		elsif(enb_f = '1') then
			reg_37_5 <= Relu_37;
		elsif(enb_g = '1') then
			reg_37_6 <= Relu_37;
		elsif(enb_h = '1') then
			reg_37_7 <= Relu_37;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_38_0 <= Relu_38;
		elsif(enb_b = '1') then
			reg_38_1 <= Relu_38;
		elsif(enb_c = '1') then
			reg_38_2 <= Relu_38;
		elsif(enb_d = '1') then
			reg_38_3 <= Relu_38;
		elsif(enb_e = '1') then
			reg_38_4 <= Relu_38;
		elsif(enb_f = '1') then
			reg_38_5 <= Relu_38;
		elsif(enb_g = '1') then
			reg_38_6 <= Relu_38;
		elsif(enb_h = '1') then
			reg_38_7 <= Relu_38;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_39_0 <= Relu_39;
		elsif(enb_b = '1') then
			reg_39_1 <= Relu_39;
		elsif(enb_c = '1') then
			reg_39_2 <= Relu_39;
		elsif(enb_d = '1') then
			reg_39_3 <= Relu_39;
		elsif(enb_e = '1') then
			reg_39_4 <= Relu_39;
		elsif(enb_f = '1') then
			reg_39_5 <= Relu_39;
		elsif(enb_g = '1') then
			reg_39_6 <= Relu_39;
		elsif(enb_h = '1') then
			reg_39_7 <= Relu_39;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_40_0 <= Relu_40;
		elsif(enb_b = '1') then
			reg_40_1 <= Relu_40;
		elsif(enb_c = '1') then
			reg_40_2 <= Relu_40;
		elsif(enb_d = '1') then
			reg_40_3 <= Relu_40;
		elsif(enb_e = '1') then
			reg_40_4 <= Relu_40;
		elsif(enb_f = '1') then
			reg_40_5 <= Relu_40;
		elsif(enb_g = '1') then
			reg_40_6 <= Relu_40;
		elsif(enb_h = '1') then
			reg_40_7 <= Relu_40;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_41_0 <= Relu_41;
		elsif(enb_b = '1') then
			reg_41_1 <= Relu_41;
		elsif(enb_c = '1') then
			reg_41_2 <= Relu_41;
		elsif(enb_d = '1') then
			reg_41_3 <= Relu_41;
		elsif(enb_e = '1') then
			reg_41_4 <= Relu_41;
		elsif(enb_f = '1') then
			reg_41_5 <= Relu_41;
		elsif(enb_g = '1') then
			reg_41_6 <= Relu_41;
		elsif(enb_h = '1') then
			reg_41_7 <= Relu_41;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_42_0 <= Relu_42;
		elsif(enb_b = '1') then
			reg_42_1 <= Relu_42;
		elsif(enb_c = '1') then
			reg_42_2 <= Relu_42;
		elsif(enb_d = '1') then
			reg_42_3 <= Relu_42;
		elsif(enb_e = '1') then
			reg_42_4 <= Relu_42;
		elsif(enb_f = '1') then
			reg_42_5 <= Relu_42;
		elsif(enb_g = '1') then
			reg_42_6 <= Relu_42;
		elsif(enb_h = '1') then
			reg_42_7 <= Relu_42;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_43_0 <= Relu_43;
		elsif(enb_b = '1') then
			reg_43_1 <= Relu_43;
		elsif(enb_c = '1') then
			reg_43_2 <= Relu_43;
		elsif(enb_d = '1') then
			reg_43_3 <= Relu_43;
		elsif(enb_e = '1') then
			reg_43_4 <= Relu_43;
		elsif(enb_f = '1') then
			reg_43_5 <= Relu_43;
		elsif(enb_g = '1') then
			reg_43_6 <= Relu_43;
		elsif(enb_h = '1') then
			reg_43_7 <= Relu_43;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_44_0 <= Relu_44;
		elsif(enb_b = '1') then
			reg_44_1 <= Relu_44;
		elsif(enb_c = '1') then
			reg_44_2 <= Relu_44;
		elsif(enb_d = '1') then
			reg_44_3 <= Relu_44;
		elsif(enb_e = '1') then
			reg_44_4 <= Relu_44;
		elsif(enb_f = '1') then
			reg_44_5 <= Relu_44;
		elsif(enb_g = '1') then
			reg_44_6 <= Relu_44;
		elsif(enb_h = '1') then
			reg_44_7 <= Relu_44;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_45_0 <= Relu_45;
		elsif(enb_b = '1') then
			reg_45_1 <= Relu_45;
		elsif(enb_c = '1') then
			reg_45_2 <= Relu_45;
		elsif(enb_d = '1') then
			reg_45_3 <= Relu_45;
		elsif(enb_e = '1') then
			reg_45_4 <= Relu_45;
		elsif(enb_f = '1') then
			reg_45_5 <= Relu_45;
		elsif(enb_g = '1') then
			reg_45_6 <= Relu_45;
		elsif(enb_h = '1') then
			reg_45_7 <= Relu_45;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_46_0 <= Relu_46;
		elsif(enb_b = '1') then
			reg_46_1 <= Relu_46;
		elsif(enb_c = '1') then
			reg_46_2 <= Relu_46;
		elsif(enb_d = '1') then
			reg_46_3 <= Relu_46;
		elsif(enb_e = '1') then
			reg_46_4 <= Relu_46;
		elsif(enb_f = '1') then
			reg_46_5 <= Relu_46;
		elsif(enb_g = '1') then
			reg_46_6 <= Relu_46;
		elsif(enb_h = '1') then
			reg_46_7 <= Relu_46;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_47_0 <= Relu_47;
		elsif(enb_b = '1') then
			reg_47_1 <= Relu_47;
		elsif(enb_c = '1') then
			reg_47_2 <= Relu_47;
		elsif(enb_d = '1') then
			reg_47_3 <= Relu_47;
		elsif(enb_e = '1') then
			reg_47_4 <= Relu_47;
		elsif(enb_f = '1') then
			reg_47_5 <= Relu_47;
		elsif(enb_g = '1') then
			reg_47_6 <= Relu_47;
		elsif(enb_h = '1') then
			reg_47_7 <= Relu_47;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_48_0 <= Relu_48;
		elsif(enb_b = '1') then
			reg_48_1 <= Relu_48;
		elsif(enb_c = '1') then
			reg_48_2 <= Relu_48;
		elsif(enb_d = '1') then
			reg_48_3 <= Relu_48;
		elsif(enb_e = '1') then
			reg_48_4 <= Relu_48;
		elsif(enb_f = '1') then
			reg_48_5 <= Relu_48;
		elsif(enb_g = '1') then
			reg_48_6 <= Relu_48;
		elsif(enb_h = '1') then
			reg_48_7 <= Relu_48;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_49_0 <= Relu_49;
		elsif(enb_b = '1') then
			reg_49_1 <= Relu_49;
		elsif(enb_c = '1') then
			reg_49_2 <= Relu_49;
		elsif(enb_d = '1') then
			reg_49_3 <= Relu_49;
		elsif(enb_e = '1') then
			reg_49_4 <= Relu_49;
		elsif(enb_f = '1') then
			reg_49_5 <= Relu_49;
		elsif(enb_g = '1') then
			reg_49_6 <= Relu_49;
		elsif(enb_h = '1') then
			reg_49_7 <= Relu_49;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_50_0 <= Relu_50;
		elsif(enb_b = '1') then
			reg_50_1 <= Relu_50;
		elsif(enb_c = '1') then
			reg_50_2 <= Relu_50;
		elsif(enb_d = '1') then
			reg_50_3 <= Relu_50;
		elsif(enb_e = '1') then
			reg_50_4 <= Relu_50;
		elsif(enb_f = '1') then
			reg_50_5 <= Relu_50;
		elsif(enb_g = '1') then
			reg_50_6 <= Relu_50;
		elsif(enb_h = '1') then
			reg_50_7 <= Relu_50;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_51_0 <= Relu_51;
		elsif(enb_b = '1') then
			reg_51_1 <= Relu_51;
		elsif(enb_c = '1') then
			reg_51_2 <= Relu_51;
		elsif(enb_d = '1') then
			reg_51_3 <= Relu_51;
		elsif(enb_e = '1') then
			reg_51_4 <= Relu_51;
		elsif(enb_f = '1') then
			reg_51_5 <= Relu_51;
		elsif(enb_g = '1') then
			reg_51_6 <= Relu_51;
		elsif(enb_h = '1') then
			reg_51_7 <= Relu_51;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_52_0 <= Relu_52;
		elsif(enb_b = '1') then
			reg_52_1 <= Relu_52;
		elsif(enb_c = '1') then
			reg_52_2 <= Relu_52;
		elsif(enb_d = '1') then
			reg_52_3 <= Relu_52;
		elsif(enb_e = '1') then
			reg_52_4 <= Relu_52;
		elsif(enb_f = '1') then
			reg_52_5 <= Relu_52;
		elsif(enb_g = '1') then
			reg_52_6 <= Relu_52;
		elsif(enb_h = '1') then
			reg_52_7 <= Relu_52;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_53_0 <= Relu_53;
		elsif(enb_b = '1') then
			reg_53_1 <= Relu_53;
		elsif(enb_c = '1') then
			reg_53_2 <= Relu_53;
		elsif(enb_d = '1') then
			reg_53_3 <= Relu_53;
		elsif(enb_e = '1') then
			reg_53_4 <= Relu_53;
		elsif(enb_f = '1') then
			reg_53_5 <= Relu_53;
		elsif(enb_g = '1') then
			reg_53_6 <= Relu_53;
		elsif(enb_h = '1') then
			reg_53_7 <= Relu_53;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_54_0 <= Relu_54;
		elsif(enb_b = '1') then
			reg_54_1 <= Relu_54;
		elsif(enb_c = '1') then
			reg_54_2 <= Relu_54;
		elsif(enb_d = '1') then
			reg_54_3 <= Relu_54;
		elsif(enb_e = '1') then
			reg_54_4 <= Relu_54;
		elsif(enb_f = '1') then
			reg_54_5 <= Relu_54;
		elsif(enb_g = '1') then
			reg_54_6 <= Relu_54;
		elsif(enb_h = '1') then
			reg_54_7 <= Relu_54;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_55_0 <= Relu_55;
		elsif(enb_b = '1') then
			reg_55_1 <= Relu_55;
		elsif(enb_c = '1') then
			reg_55_2 <= Relu_55;
		elsif(enb_d = '1') then
			reg_55_3 <= Relu_55;
		elsif(enb_e = '1') then
			reg_55_4 <= Relu_55;
		elsif(enb_f = '1') then
			reg_55_5 <= Relu_55;
		elsif(enb_g = '1') then
			reg_55_6 <= Relu_55;
		elsif(enb_h = '1') then
			reg_55_7 <= Relu_55;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_56_0 <= Relu_56;
		elsif(enb_b = '1') then
			reg_56_1 <= Relu_56;
		elsif(enb_c = '1') then
			reg_56_2 <= Relu_56;
		elsif(enb_d = '1') then
			reg_56_3 <= Relu_56;
		elsif(enb_e = '1') then
			reg_56_4 <= Relu_56;
		elsif(enb_f = '1') then
			reg_56_5 <= Relu_56;
		elsif(enb_g = '1') then
			reg_56_6 <= Relu_56;
		elsif(enb_h = '1') then
			reg_56_7 <= Relu_56;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_57_0 <= Relu_57;
		elsif(enb_b = '1') then
			reg_57_1 <= Relu_57;
		elsif(enb_c = '1') then
			reg_57_2 <= Relu_57;
		elsif(enb_d = '1') then
			reg_57_3 <= Relu_57;
		elsif(enb_e = '1') then
			reg_57_4 <= Relu_57;
		elsif(enb_f = '1') then
			reg_57_5 <= Relu_57;
		elsif(enb_g = '1') then
			reg_57_6 <= Relu_57;
		elsif(enb_h = '1') then
			reg_57_7 <= Relu_57;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_58_0 <= Relu_58;
		elsif(enb_b = '1') then
			reg_58_1 <= Relu_58;
		elsif(enb_c = '1') then
			reg_58_2 <= Relu_58;
		elsif(enb_d = '1') then
			reg_58_3 <= Relu_58;
		elsif(enb_e = '1') then
			reg_58_4 <= Relu_58;
		elsif(enb_f = '1') then
			reg_58_5 <= Relu_58;
		elsif(enb_g = '1') then
			reg_58_6 <= Relu_58;
		elsif(enb_h = '1') then
			reg_58_7 <= Relu_58;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_59_0 <= Relu_59;
		elsif(enb_b = '1') then
			reg_59_1 <= Relu_59;
		elsif(enb_c = '1') then
			reg_59_2 <= Relu_59;
		elsif(enb_d = '1') then
			reg_59_3 <= Relu_59;
		elsif(enb_e = '1') then
			reg_59_4 <= Relu_59;
		elsif(enb_f = '1') then
			reg_59_5 <= Relu_59;
		elsif(enb_g = '1') then
			reg_59_6 <= Relu_59;
		elsif(enb_h = '1') then
			reg_59_7 <= Relu_59;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_60_0 <= Relu_60;
		elsif(enb_b = '1') then
			reg_60_1 <= Relu_60;
		elsif(enb_c = '1') then
			reg_60_2 <= Relu_60;
		elsif(enb_d = '1') then
			reg_60_3 <= Relu_60;
		elsif(enb_e = '1') then
			reg_60_4 <= Relu_60;
		elsif(enb_f = '1') then
			reg_60_5 <= Relu_60;
		elsif(enb_g = '1') then
			reg_60_6 <= Relu_60;
		elsif(enb_h = '1') then
			reg_60_7 <= Relu_60;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_61_0 <= Relu_61;
		elsif(enb_b = '1') then
			reg_61_1 <= Relu_61;
		elsif(enb_c = '1') then
			reg_61_2 <= Relu_61;
		elsif(enb_d = '1') then
			reg_61_3 <= Relu_61;
		elsif(enb_e = '1') then
			reg_61_4 <= Relu_61;
		elsif(enb_f = '1') then
			reg_61_5 <= Relu_61;
		elsif(enb_g = '1') then
			reg_61_6 <= Relu_61;
		elsif(enb_h = '1') then
			reg_61_7 <= Relu_61;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_62_0 <= Relu_62;
		elsif(enb_b = '1') then
			reg_62_1 <= Relu_62;
		elsif(enb_c = '1') then
			reg_62_2 <= Relu_62;
		elsif(enb_d = '1') then
			reg_62_3 <= Relu_62;
		elsif(enb_e = '1') then
			reg_62_4 <= Relu_62;
		elsif(enb_f = '1') then
			reg_62_5 <= Relu_62;
		elsif(enb_g = '1') then
			reg_62_6 <= Relu_62;
		elsif(enb_h = '1') then
			reg_62_7 <= Relu_62;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_63_0 <= Relu_63;
		elsif(enb_b = '1') then
			reg_63_1 <= Relu_63;
		elsif(enb_c = '1') then
			reg_63_2 <= Relu_63;
		elsif(enb_d = '1') then
			reg_63_3 <= Relu_63;
		elsif(enb_e = '1') then
			reg_63_4 <= Relu_63;
		elsif(enb_f = '1') then
			reg_63_5 <= Relu_63;
		elsif(enb_g = '1') then
			reg_63_6 <= Relu_63;
		elsif(enb_h = '1') then
			reg_63_7 <= Relu_63;
		end if;
	end if;  
end process;


---------------------------------------------------------
ROM_W8_inst : ROM_W8 port map (
  CLK => CLK,
  ADDR => W8Addr,
  DATA => W8Data );

---------------------------------------------------------

coeff_00 <= W8Data( 1535 downto 1533 ) when W8Sel = "000" else W8Data( 1532 downto 1530 ) when W8Sel = "001" else W8Data( 1529 downto 1527 ) when W8Sel = "010" else W8Data( 1526 downto 1524 ) when W8Sel = "011" else W8Data( 1523 downto 1521 ) when W8Sel = "100" else W8Data( 1520 downto 1518 ) when W8Sel = "101" else W8Data( 1517 downto 1515 ) when W8Sel = "110" else W8Data( 1514 downto 1512 );

coeff_01 <= W8Data( 1511 downto 1509 ) when W8Sel = "000" else W8Data( 1508 downto 1506 ) when W8Sel = "001" else W8Data( 1505 downto 1503 ) when W8Sel = "010" else W8Data( 1502 downto 1500 ) when W8Sel = "011" else W8Data( 1499 downto 1497 ) when W8Sel = "100" else W8Data( 1496 downto 1494 ) when W8Sel = "101" else W8Data( 1493 downto 1491 ) when W8Sel = "110" else W8Data( 1490 downto 1488 );

coeff_02 <= W8Data( 1487 downto 1485 ) when W8Sel = "000" else W8Data( 1484 downto 1482 ) when W8Sel = "001" else W8Data( 1481 downto 1479 ) when W8Sel = "010" else W8Data( 1478 downto 1476 ) when W8Sel = "011" else W8Data( 1475 downto 1473 ) when W8Sel = "100" else W8Data( 1472 downto 1470 ) when W8Sel = "101" else W8Data( 1469 downto 1467 ) when W8Sel = "110" else W8Data( 1466 downto 1464 );

coeff_03 <= W8Data( 1463 downto 1461 ) when W8Sel = "000" else W8Data( 1460 downto 1458 ) when W8Sel = "001" else W8Data( 1457 downto 1455 ) when W8Sel = "010" else W8Data( 1454 downto 1452 ) when W8Sel = "011" else W8Data( 1451 downto 1449 ) when W8Sel = "100" else W8Data( 1448 downto 1446 ) when W8Sel = "101" else W8Data( 1445 downto 1443 ) when W8Sel = "110" else W8Data( 1442 downto 1440 );

coeff_04 <= W8Data( 1439 downto 1437 ) when W8Sel = "000" else W8Data( 1436 downto 1434 ) when W8Sel = "001" else W8Data( 1433 downto 1431 ) when W8Sel = "010" else W8Data( 1430 downto 1428 ) when W8Sel = "011" else W8Data( 1427 downto 1425 ) when W8Sel = "100" else W8Data( 1424 downto 1422 ) when W8Sel = "101" else W8Data( 1421 downto 1419 ) when W8Sel = "110" else W8Data( 1418 downto 1416 );

coeff_05 <= W8Data( 1415 downto 1413 ) when W8Sel = "000" else W8Data( 1412 downto 1410 ) when W8Sel = "001" else W8Data( 1409 downto 1407 ) when W8Sel = "010" else W8Data( 1406 downto 1404 ) when W8Sel = "011" else W8Data( 1403 downto 1401 ) when W8Sel = "100" else W8Data( 1400 downto 1398 ) when W8Sel = "101" else W8Data( 1397 downto 1395 ) when W8Sel = "110" else W8Data( 1394 downto 1392 );

coeff_06 <= W8Data( 1391 downto 1389 ) when W8Sel = "000" else W8Data( 1388 downto 1386 ) when W8Sel = "001" else W8Data( 1385 downto 1383 ) when W8Sel = "010" else W8Data( 1382 downto 1380 ) when W8Sel = "011" else W8Data( 1379 downto 1377 ) when W8Sel = "100" else W8Data( 1376 downto 1374 ) when W8Sel = "101" else W8Data( 1373 downto 1371 ) when W8Sel = "110" else W8Data( 1370 downto 1368 );

coeff_07 <= W8Data( 1367 downto 1365 ) when W8Sel = "000" else W8Data( 1364 downto 1362 ) when W8Sel = "001" else W8Data( 1361 downto 1359 ) when W8Sel = "010" else W8Data( 1358 downto 1356 ) when W8Sel = "011" else W8Data( 1355 downto 1353 ) when W8Sel = "100" else W8Data( 1352 downto 1350 ) when W8Sel = "101" else W8Data( 1349 downto 1347 ) when W8Sel = "110" else W8Data( 1346 downto 1344 );

coeff_08 <= W8Data( 1343 downto 1341 ) when W8Sel = "000" else W8Data( 1340 downto 1338 ) when W8Sel = "001" else W8Data( 1337 downto 1335 ) when W8Sel = "010" else W8Data( 1334 downto 1332 ) when W8Sel = "011" else W8Data( 1331 downto 1329 ) when W8Sel = "100" else W8Data( 1328 downto 1326 ) when W8Sel = "101" else W8Data( 1325 downto 1323 ) when W8Sel = "110" else W8Data( 1322 downto 1320 );

coeff_09 <= W8Data( 1319 downto 1317 ) when W8Sel = "000" else W8Data( 1316 downto 1314 ) when W8Sel = "001" else W8Data( 1313 downto 1311 ) when W8Sel = "010" else W8Data( 1310 downto 1308 ) when W8Sel = "011" else W8Data( 1307 downto 1305 ) when W8Sel = "100" else W8Data( 1304 downto 1302 ) when W8Sel = "101" else W8Data( 1301 downto 1299 ) when W8Sel = "110" else W8Data( 1298 downto 1296 );

coeff_10 <= W8Data( 1295 downto 1293 ) when W8Sel = "000" else W8Data( 1292 downto 1290 ) when W8Sel = "001" else W8Data( 1289 downto 1287 ) when W8Sel = "010" else W8Data( 1286 downto 1284 ) when W8Sel = "011" else W8Data( 1283 downto 1281 ) when W8Sel = "100" else W8Data( 1280 downto 1278 ) when W8Sel = "101" else W8Data( 1277 downto 1275 ) when W8Sel = "110" else W8Data( 1274 downto 1272 );

coeff_11 <= W8Data( 1271 downto 1269 ) when W8Sel = "000" else W8Data( 1268 downto 1266 ) when W8Sel = "001" else W8Data( 1265 downto 1263 ) when W8Sel = "010" else W8Data( 1262 downto 1260 ) when W8Sel = "011" else W8Data( 1259 downto 1257 ) when W8Sel = "100" else W8Data( 1256 downto 1254 ) when W8Sel = "101" else W8Data( 1253 downto 1251 ) when W8Sel = "110" else W8Data( 1250 downto 1248 );

coeff_12 <= W8Data( 1247 downto 1245 ) when W8Sel = "000" else W8Data( 1244 downto 1242 ) when W8Sel = "001" else W8Data( 1241 downto 1239 ) when W8Sel = "010" else W8Data( 1238 downto 1236 ) when W8Sel = "011" else W8Data( 1235 downto 1233 ) when W8Sel = "100" else W8Data( 1232 downto 1230 ) when W8Sel = "101" else W8Data( 1229 downto 1227 ) when W8Sel = "110" else W8Data( 1226 downto 1224 );

coeff_13 <= W8Data( 1223 downto 1221 ) when W8Sel = "000" else W8Data( 1220 downto 1218 ) when W8Sel = "001" else W8Data( 1217 downto 1215 ) when W8Sel = "010" else W8Data( 1214 downto 1212 ) when W8Sel = "011" else W8Data( 1211 downto 1209 ) when W8Sel = "100" else W8Data( 1208 downto 1206 ) when W8Sel = "101" else W8Data( 1205 downto 1203 ) when W8Sel = "110" else W8Data( 1202 downto 1200 );

coeff_14 <= W8Data( 1199 downto 1197 ) when W8Sel = "000" else W8Data( 1196 downto 1194 ) when W8Sel = "001" else W8Data( 1193 downto 1191 ) when W8Sel = "010" else W8Data( 1190 downto 1188 ) when W8Sel = "011" else W8Data( 1187 downto 1185 ) when W8Sel = "100" else W8Data( 1184 downto 1182 ) when W8Sel = "101" else W8Data( 1181 downto 1179 ) when W8Sel = "110" else W8Data( 1178 downto 1176 );

coeff_15 <= W8Data( 1175 downto 1173 ) when W8Sel = "000" else W8Data( 1172 downto 1170 ) when W8Sel = "001" else W8Data( 1169 downto 1167 ) when W8Sel = "010" else W8Data( 1166 downto 1164 ) when W8Sel = "011" else W8Data( 1163 downto 1161 ) when W8Sel = "100" else W8Data( 1160 downto 1158 ) when W8Sel = "101" else W8Data( 1157 downto 1155 ) when W8Sel = "110" else W8Data( 1154 downto 1152 );

coeff_16 <= W8Data( 1151 downto 1149 ) when W8Sel = "000" else W8Data( 1148 downto 1146 ) when W8Sel = "001" else W8Data( 1145 downto 1143 ) when W8Sel = "010" else W8Data( 1142 downto 1140 ) when W8Sel = "011" else W8Data( 1139 downto 1137 ) when W8Sel = "100" else W8Data( 1136 downto 1134 ) when W8Sel = "101" else W8Data( 1133 downto 1131 ) when W8Sel = "110" else W8Data( 1130 downto 1128 );

coeff_17 <= W8Data( 1127 downto 1125 ) when W8Sel = "000" else W8Data( 1124 downto 1122 ) when W8Sel = "001" else W8Data( 1121 downto 1119 ) when W8Sel = "010" else W8Data( 1118 downto 1116 ) when W8Sel = "011" else W8Data( 1115 downto 1113 ) when W8Sel = "100" else W8Data( 1112 downto 1110 ) when W8Sel = "101" else W8Data( 1109 downto 1107 ) when W8Sel = "110" else W8Data( 1106 downto 1104 );

coeff_18 <= W8Data( 1103 downto 1101 ) when W8Sel = "000" else W8Data( 1100 downto 1098 ) when W8Sel = "001" else W8Data( 1097 downto 1095 ) when W8Sel = "010" else W8Data( 1094 downto 1092 ) when W8Sel = "011" else W8Data( 1091 downto 1089 ) when W8Sel = "100" else W8Data( 1088 downto 1086 ) when W8Sel = "101" else W8Data( 1085 downto 1083 ) when W8Sel = "110" else W8Data( 1082 downto 1080 );

coeff_19 <= W8Data( 1079 downto 1077 ) when W8Sel = "000" else W8Data( 1076 downto 1074 ) when W8Sel = "001" else W8Data( 1073 downto 1071 ) when W8Sel = "010" else W8Data( 1070 downto 1068 ) when W8Sel = "011" else W8Data( 1067 downto 1065 ) when W8Sel = "100" else W8Data( 1064 downto 1062 ) when W8Sel = "101" else W8Data( 1061 downto 1059 ) when W8Sel = "110" else W8Data( 1058 downto 1056 );

coeff_20 <= W8Data( 1055 downto 1053 ) when W8Sel = "000" else W8Data( 1052 downto 1050 ) when W8Sel = "001" else W8Data( 1049 downto 1047 ) when W8Sel = "010" else W8Data( 1046 downto 1044 ) when W8Sel = "011" else W8Data( 1043 downto 1041 ) when W8Sel = "100" else W8Data( 1040 downto 1038 ) when W8Sel = "101" else W8Data( 1037 downto 1035 ) when W8Sel = "110" else W8Data( 1034 downto 1032 );

coeff_21 <= W8Data( 1031 downto 1029 ) when W8Sel = "000" else W8Data( 1028 downto 1026 ) when W8Sel = "001" else W8Data( 1025 downto 1023 ) when W8Sel = "010" else W8Data( 1022 downto 1020 ) when W8Sel = "011" else W8Data( 1019 downto 1017 ) when W8Sel = "100" else W8Data( 1016 downto 1014 ) when W8Sel = "101" else W8Data( 1013 downto 1011 ) when W8Sel = "110" else W8Data( 1010 downto 1008 );

coeff_22 <= W8Data( 1007 downto 1005 ) when W8Sel = "000" else W8Data( 1004 downto 1002 ) when W8Sel = "001" else W8Data( 1001 downto 999 ) when W8Sel = "010" else W8Data( 998 downto 996 ) when W8Sel = "011" else W8Data( 995 downto 993 ) when W8Sel = "100" else W8Data( 992 downto 990 ) when W8Sel = "101" else W8Data( 989 downto 987 ) when W8Sel = "110" else W8Data( 986 downto 984 );

coeff_23 <= W8Data( 983 downto 981 ) when W8Sel = "000" else W8Data( 980 downto 978 ) when W8Sel = "001" else W8Data( 977 downto 975 ) when W8Sel = "010" else W8Data( 974 downto 972 ) when W8Sel = "011" else W8Data( 971 downto 969 ) when W8Sel = "100" else W8Data( 968 downto 966 ) when W8Sel = "101" else W8Data( 965 downto 963 ) when W8Sel = "110" else W8Data( 962 downto 960 );

coeff_24 <= W8Data( 959 downto 957 ) when W8Sel = "000" else W8Data( 956 downto 954 ) when W8Sel = "001" else W8Data( 953 downto 951 ) when W8Sel = "010" else W8Data( 950 downto 948 ) when W8Sel = "011" else W8Data( 947 downto 945 ) when W8Sel = "100" else W8Data( 944 downto 942 ) when W8Sel = "101" else W8Data( 941 downto 939 ) when W8Sel = "110" else W8Data( 938 downto 936 );

coeff_25 <= W8Data( 935 downto 933 ) when W8Sel = "000" else W8Data( 932 downto 930 ) when W8Sel = "001" else W8Data( 929 downto 927 ) when W8Sel = "010" else W8Data( 926 downto 924 ) when W8Sel = "011" else W8Data( 923 downto 921 ) when W8Sel = "100" else W8Data( 920 downto 918 ) when W8Sel = "101" else W8Data( 917 downto 915 ) when W8Sel = "110" else W8Data( 914 downto 912 );

coeff_26 <= W8Data( 911 downto 909 ) when W8Sel = "000" else W8Data( 908 downto 906 ) when W8Sel = "001" else W8Data( 905 downto 903 ) when W8Sel = "010" else W8Data( 902 downto 900 ) when W8Sel = "011" else W8Data( 899 downto 897 ) when W8Sel = "100" else W8Data( 896 downto 894 ) when W8Sel = "101" else W8Data( 893 downto 891 ) when W8Sel = "110" else W8Data( 890 downto 888 );

coeff_27 <= W8Data( 887 downto 885 ) when W8Sel = "000" else W8Data( 884 downto 882 ) when W8Sel = "001" else W8Data( 881 downto 879 ) when W8Sel = "010" else W8Data( 878 downto 876 ) when W8Sel = "011" else W8Data( 875 downto 873 ) when W8Sel = "100" else W8Data( 872 downto 870 ) when W8Sel = "101" else W8Data( 869 downto 867 ) when W8Sel = "110" else W8Data( 866 downto 864 );

coeff_28 <= W8Data( 863 downto 861 ) when W8Sel = "000" else W8Data( 860 downto 858 ) when W8Sel = "001" else W8Data( 857 downto 855 ) when W8Sel = "010" else W8Data( 854 downto 852 ) when W8Sel = "011" else W8Data( 851 downto 849 ) when W8Sel = "100" else W8Data( 848 downto 846 ) when W8Sel = "101" else W8Data( 845 downto 843 ) when W8Sel = "110" else W8Data( 842 downto 840 );

coeff_29 <= W8Data( 839 downto 837 ) when W8Sel = "000" else W8Data( 836 downto 834 ) when W8Sel = "001" else W8Data( 833 downto 831 ) when W8Sel = "010" else W8Data( 830 downto 828 ) when W8Sel = "011" else W8Data( 827 downto 825 ) when W8Sel = "100" else W8Data( 824 downto 822 ) when W8Sel = "101" else W8Data( 821 downto 819 ) when W8Sel = "110" else W8Data( 818 downto 816 );

coeff_30 <= W8Data( 815 downto 813 ) when W8Sel = "000" else W8Data( 812 downto 810 ) when W8Sel = "001" else W8Data( 809 downto 807 ) when W8Sel = "010" else W8Data( 806 downto 804 ) when W8Sel = "011" else W8Data( 803 downto 801 ) when W8Sel = "100" else W8Data( 800 downto 798 ) when W8Sel = "101" else W8Data( 797 downto 795 ) when W8Sel = "110" else W8Data( 794 downto 792 );

coeff_31 <= W8Data( 791 downto 789 ) when W8Sel = "000" else W8Data( 788 downto 786 ) when W8Sel = "001" else W8Data( 785 downto 783 ) when W8Sel = "010" else W8Data( 782 downto 780 ) when W8Sel = "011" else W8Data( 779 downto 777 ) when W8Sel = "100" else W8Data( 776 downto 774 ) when W8Sel = "101" else W8Data( 773 downto 771 ) when W8Sel = "110" else W8Data( 770 downto 768 );

coeff_32 <= W8Data( 767 downto 765 ) when W8Sel = "000" else W8Data( 764 downto 762 ) when W8Sel = "001" else W8Data( 761 downto 759 ) when W8Sel = "010" else W8Data( 758 downto 756 ) when W8Sel = "011" else W8Data( 755 downto 753 ) when W8Sel = "100" else W8Data( 752 downto 750 ) when W8Sel = "101" else W8Data( 749 downto 747 ) when W8Sel = "110" else W8Data( 746 downto 744 );

coeff_33 <= W8Data( 743 downto 741 ) when W8Sel = "000" else W8Data( 740 downto 738 ) when W8Sel = "001" else W8Data( 737 downto 735 ) when W8Sel = "010" else W8Data( 734 downto 732 ) when W8Sel = "011" else W8Data( 731 downto 729 ) when W8Sel = "100" else W8Data( 728 downto 726 ) when W8Sel = "101" else W8Data( 725 downto 723 ) when W8Sel = "110" else W8Data( 722 downto 720 );

coeff_34 <= W8Data( 719 downto 717 ) when W8Sel = "000" else W8Data( 716 downto 714 ) when W8Sel = "001" else W8Data( 713 downto 711 ) when W8Sel = "010" else W8Data( 710 downto 708 ) when W8Sel = "011" else W8Data( 707 downto 705 ) when W8Sel = "100" else W8Data( 704 downto 702 ) when W8Sel = "101" else W8Data( 701 downto 699 ) when W8Sel = "110" else W8Data( 698 downto 696 );

coeff_35 <= W8Data( 695 downto 693 ) when W8Sel = "000" else W8Data( 692 downto 690 ) when W8Sel = "001" else W8Data( 689 downto 687 ) when W8Sel = "010" else W8Data( 686 downto 684 ) when W8Sel = "011" else W8Data( 683 downto 681 ) when W8Sel = "100" else W8Data( 680 downto 678 ) when W8Sel = "101" else W8Data( 677 downto 675 ) when W8Sel = "110" else W8Data( 674 downto 672 );

coeff_36 <= W8Data( 671 downto 669 ) when W8Sel = "000" else W8Data( 668 downto 666 ) when W8Sel = "001" else W8Data( 665 downto 663 ) when W8Sel = "010" else W8Data( 662 downto 660 ) when W8Sel = "011" else W8Data( 659 downto 657 ) when W8Sel = "100" else W8Data( 656 downto 654 ) when W8Sel = "101" else W8Data( 653 downto 651 ) when W8Sel = "110" else W8Data( 650 downto 648 );

coeff_37 <= W8Data( 647 downto 645 ) when W8Sel = "000" else W8Data( 644 downto 642 ) when W8Sel = "001" else W8Data( 641 downto 639 ) when W8Sel = "010" else W8Data( 638 downto 636 ) when W8Sel = "011" else W8Data( 635 downto 633 ) when W8Sel = "100" else W8Data( 632 downto 630 ) when W8Sel = "101" else W8Data( 629 downto 627 ) when W8Sel = "110" else W8Data( 626 downto 624 );

coeff_38 <= W8Data( 623 downto 621 ) when W8Sel = "000" else W8Data( 620 downto 618 ) when W8Sel = "001" else W8Data( 617 downto 615 ) when W8Sel = "010" else W8Data( 614 downto 612 ) when W8Sel = "011" else W8Data( 611 downto 609 ) when W8Sel = "100" else W8Data( 608 downto 606 ) when W8Sel = "101" else W8Data( 605 downto 603 ) when W8Sel = "110" else W8Data( 602 downto 600 );

coeff_39 <= W8Data( 599 downto 597 ) when W8Sel = "000" else W8Data( 596 downto 594 ) when W8Sel = "001" else W8Data( 593 downto 591 ) when W8Sel = "010" else W8Data( 590 downto 588 ) when W8Sel = "011" else W8Data( 587 downto 585 ) when W8Sel = "100" else W8Data( 584 downto 582 ) when W8Sel = "101" else W8Data( 581 downto 579 ) when W8Sel = "110" else W8Data( 578 downto 576 );

coeff_40 <= W8Data( 575 downto 573 ) when W8Sel = "000" else W8Data( 572 downto 570 ) when W8Sel = "001" else W8Data( 569 downto 567 ) when W8Sel = "010" else W8Data( 566 downto 564 ) when W8Sel = "011" else W8Data( 563 downto 561 ) when W8Sel = "100" else W8Data( 560 downto 558 ) when W8Sel = "101" else W8Data( 557 downto 555 ) when W8Sel = "110" else W8Data( 554 downto 552 );

coeff_41 <= W8Data( 551 downto 549 ) when W8Sel = "000" else W8Data( 548 downto 546 ) when W8Sel = "001" else W8Data( 545 downto 543 ) when W8Sel = "010" else W8Data( 542 downto 540 ) when W8Sel = "011" else W8Data( 539 downto 537 ) when W8Sel = "100" else W8Data( 536 downto 534 ) when W8Sel = "101" else W8Data( 533 downto 531 ) when W8Sel = "110" else W8Data( 530 downto 528 );

coeff_42 <= W8Data( 527 downto 525 ) when W8Sel = "000" else W8Data( 524 downto 522 ) when W8Sel = "001" else W8Data( 521 downto 519 ) when W8Sel = "010" else W8Data( 518 downto 516 ) when W8Sel = "011" else W8Data( 515 downto 513 ) when W8Sel = "100" else W8Data( 512 downto 510 ) when W8Sel = "101" else W8Data( 509 downto 507 ) when W8Sel = "110" else W8Data( 506 downto 504 );

coeff_43 <= W8Data( 503 downto 501 ) when W8Sel = "000" else W8Data( 500 downto 498 ) when W8Sel = "001" else W8Data( 497 downto 495 ) when W8Sel = "010" else W8Data( 494 downto 492 ) when W8Sel = "011" else W8Data( 491 downto 489 ) when W8Sel = "100" else W8Data( 488 downto 486 ) when W8Sel = "101" else W8Data( 485 downto 483 ) when W8Sel = "110" else W8Data( 482 downto 480 );

coeff_44 <= W8Data( 479 downto 477 ) when W8Sel = "000" else W8Data( 476 downto 474 ) when W8Sel = "001" else W8Data( 473 downto 471 ) when W8Sel = "010" else W8Data( 470 downto 468 ) when W8Sel = "011" else W8Data( 467 downto 465 ) when W8Sel = "100" else W8Data( 464 downto 462 ) when W8Sel = "101" else W8Data( 461 downto 459 ) when W8Sel = "110" else W8Data( 458 downto 456 );

coeff_45 <= W8Data( 455 downto 453 ) when W8Sel = "000" else W8Data( 452 downto 450 ) when W8Sel = "001" else W8Data( 449 downto 447 ) when W8Sel = "010" else W8Data( 446 downto 444 ) when W8Sel = "011" else W8Data( 443 downto 441 ) when W8Sel = "100" else W8Data( 440 downto 438 ) when W8Sel = "101" else W8Data( 437 downto 435 ) when W8Sel = "110" else W8Data( 434 downto 432 );

coeff_46 <= W8Data( 431 downto 429 ) when W8Sel = "000" else W8Data( 428 downto 426 ) when W8Sel = "001" else W8Data( 425 downto 423 ) when W8Sel = "010" else W8Data( 422 downto 420 ) when W8Sel = "011" else W8Data( 419 downto 417 ) when W8Sel = "100" else W8Data( 416 downto 414 ) when W8Sel = "101" else W8Data( 413 downto 411 ) when W8Sel = "110" else W8Data( 410 downto 408 );

coeff_47 <= W8Data( 407 downto 405 ) when W8Sel = "000" else W8Data( 404 downto 402 ) when W8Sel = "001" else W8Data( 401 downto 399 ) when W8Sel = "010" else W8Data( 398 downto 396 ) when W8Sel = "011" else W8Data( 395 downto 393 ) when W8Sel = "100" else W8Data( 392 downto 390 ) when W8Sel = "101" else W8Data( 389 downto 387 ) when W8Sel = "110" else W8Data( 386 downto 384 );

coeff_48 <= W8Data( 383 downto 381 ) when W8Sel = "000" else W8Data( 380 downto 378 ) when W8Sel = "001" else W8Data( 377 downto 375 ) when W8Sel = "010" else W8Data( 374 downto 372 ) when W8Sel = "011" else W8Data( 371 downto 369 ) when W8Sel = "100" else W8Data( 368 downto 366 ) when W8Sel = "101" else W8Data( 365 downto 363 ) when W8Sel = "110" else W8Data( 362 downto 360 );

coeff_49 <= W8Data( 359 downto 357 ) when W8Sel = "000" else W8Data( 356 downto 354 ) when W8Sel = "001" else W8Data( 353 downto 351 ) when W8Sel = "010" else W8Data( 350 downto 348 ) when W8Sel = "011" else W8Data( 347 downto 345 ) when W8Sel = "100" else W8Data( 344 downto 342 ) when W8Sel = "101" else W8Data( 341 downto 339 ) when W8Sel = "110" else W8Data( 338 downto 336 );

coeff_50 <= W8Data( 335 downto 333 ) when W8Sel = "000" else W8Data( 332 downto 330 ) when W8Sel = "001" else W8Data( 329 downto 327 ) when W8Sel = "010" else W8Data( 326 downto 324 ) when W8Sel = "011" else W8Data( 323 downto 321 ) when W8Sel = "100" else W8Data( 320 downto 318 ) when W8Sel = "101" else W8Data( 317 downto 315 ) when W8Sel = "110" else W8Data( 314 downto 312 );

coeff_51 <= W8Data( 311 downto 309 ) when W8Sel = "000" else W8Data( 308 downto 306 ) when W8Sel = "001" else W8Data( 305 downto 303 ) when W8Sel = "010" else W8Data( 302 downto 300 ) when W8Sel = "011" else W8Data( 299 downto 297 ) when W8Sel = "100" else W8Data( 296 downto 294 ) when W8Sel = "101" else W8Data( 293 downto 291 ) when W8Sel = "110" else W8Data( 290 downto 288 );

coeff_52 <= W8Data( 287 downto 285 ) when W8Sel = "000" else W8Data( 284 downto 282 ) when W8Sel = "001" else W8Data( 281 downto 279 ) when W8Sel = "010" else W8Data( 278 downto 276 ) when W8Sel = "011" else W8Data( 275 downto 273 ) when W8Sel = "100" else W8Data( 272 downto 270 ) when W8Sel = "101" else W8Data( 269 downto 267 ) when W8Sel = "110" else W8Data( 266 downto 264 );

coeff_53 <= W8Data( 263 downto 261 ) when W8Sel = "000" else W8Data( 260 downto 258 ) when W8Sel = "001" else W8Data( 257 downto 255 ) when W8Sel = "010" else W8Data( 254 downto 252 ) when W8Sel = "011" else W8Data( 251 downto 249 ) when W8Sel = "100" else W8Data( 248 downto 246 ) when W8Sel = "101" else W8Data( 245 downto 243 ) when W8Sel = "110" else W8Data( 242 downto 240 );

coeff_54 <= W8Data( 239 downto 237 ) when W8Sel = "000" else W8Data( 236 downto 234 ) when W8Sel = "001" else W8Data( 233 downto 231 ) when W8Sel = "010" else W8Data( 230 downto 228 ) when W8Sel = "011" else W8Data( 227 downto 225 ) when W8Sel = "100" else W8Data( 224 downto 222 ) when W8Sel = "101" else W8Data( 221 downto 219 ) when W8Sel = "110" else W8Data( 218 downto 216 );

coeff_55 <= W8Data( 215 downto 213 ) when W8Sel = "000" else W8Data( 212 downto 210 ) when W8Sel = "001" else W8Data( 209 downto 207 ) when W8Sel = "010" else W8Data( 206 downto 204 ) when W8Sel = "011" else W8Data( 203 downto 201 ) when W8Sel = "100" else W8Data( 200 downto 198 ) when W8Sel = "101" else W8Data( 197 downto 195 ) when W8Sel = "110" else W8Data( 194 downto 192 );

coeff_56 <= W8Data( 191 downto 189 ) when W8Sel = "000" else W8Data( 188 downto 186 ) when W8Sel = "001" else W8Data( 185 downto 183 ) when W8Sel = "010" else W8Data( 182 downto 180 ) when W8Sel = "011" else W8Data( 179 downto 177 ) when W8Sel = "100" else W8Data( 176 downto 174 ) when W8Sel = "101" else W8Data( 173 downto 171 ) when W8Sel = "110" else W8Data( 170 downto 168 );

coeff_57 <= W8Data( 167 downto 165 ) when W8Sel = "000" else W8Data( 164 downto 162 ) when W8Sel = "001" else W8Data( 161 downto 159 ) when W8Sel = "010" else W8Data( 158 downto 156 ) when W8Sel = "011" else W8Data( 155 downto 153 ) when W8Sel = "100" else W8Data( 152 downto 150 ) when W8Sel = "101" else W8Data( 149 downto 147 ) when W8Sel = "110" else W8Data( 146 downto 144 );

coeff_58 <= W8Data( 143 downto 141 ) when W8Sel = "000" else W8Data( 140 downto 138 ) when W8Sel = "001" else W8Data( 137 downto 135 ) when W8Sel = "010" else W8Data( 134 downto 132 ) when W8Sel = "011" else W8Data( 131 downto 129 ) when W8Sel = "100" else W8Data( 128 downto 126 ) when W8Sel = "101" else W8Data( 125 downto 123 ) when W8Sel = "110" else W8Data( 122 downto 120 );

coeff_59 <= W8Data( 119 downto 117 ) when W8Sel = "000" else W8Data( 116 downto 114 ) when W8Sel = "001" else W8Data( 113 downto 111 ) when W8Sel = "010" else W8Data( 110 downto 108 ) when W8Sel = "011" else W8Data( 107 downto 105 ) when W8Sel = "100" else W8Data( 104 downto 102 ) when W8Sel = "101" else W8Data( 101 downto 99 ) when W8Sel = "110" else W8Data( 98 downto 96 );

coeff_60 <= W8Data( 95 downto 93 ) when W8Sel = "000" else W8Data( 92 downto 90 ) when W8Sel = "001" else W8Data( 89 downto 87 ) when W8Sel = "010" else W8Data( 86 downto 84 ) when W8Sel = "011" else W8Data( 83 downto 81 ) when W8Sel = "100" else W8Data( 80 downto 78 ) when W8Sel = "101" else W8Data( 77 downto 75 ) when W8Sel = "110" else W8Data( 74 downto 72 );

coeff_61 <= W8Data( 71 downto 69 ) when W8Sel = "000" else W8Data( 68 downto 66 ) when W8Sel = "001" else W8Data( 65 downto 63 ) when W8Sel = "010" else W8Data( 62 downto 60 ) when W8Sel = "011" else W8Data( 59 downto 57 ) when W8Sel = "100" else W8Data( 56 downto 54 ) when W8Sel = "101" else W8Data( 53 downto 51 ) when W8Sel = "110" else W8Data( 50 downto 48 );

coeff_62 <= W8Data( 47 downto 45 ) when W8Sel = "000" else W8Data( 44 downto 42 ) when W8Sel = "001" else W8Data( 41 downto 39 ) when W8Sel = "010" else W8Data( 38 downto 36 ) when W8Sel = "011" else W8Data( 35 downto 33 ) when W8Sel = "100" else W8Data( 32 downto 30 ) when W8Sel = "101" else W8Data( 29 downto 27 ) when W8Sel = "110" else W8Data( 26 downto 24 );

coeff_63 <= W8Data( 23 downto 21 ) when W8Sel = "000" else W8Data( 20 downto 18 ) when W8Sel = "001" else W8Data( 17 downto 15 ) when W8Sel = "010" else W8Data( 14 downto 12 ) when W8Sel = "011" else W8Data( 11 downto 9 ) when W8Sel = "100" else W8Data( 8 downto 6 ) when W8Sel = "101" else W8Data( 5 downto 3 ) when W8Sel = "110" else W8Data( 2 downto 0 );

---------------------------------------------------------
dotpro_w8_00 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_00,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_00 );

dotpro_w8_01 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_01,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_01 );

dotpro_w8_02 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_02,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_02 );

dotpro_w8_03 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_03,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_03 );

dotpro_w8_04 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_04,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_04 );

dotpro_w8_05 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_05,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_05 );

dotpro_w8_06 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_06,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_06 );

dotpro_w8_07 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_07,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_07 );

dotpro_w8_08 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_08,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_08 );

dotpro_w8_09 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_09,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_09 );

---------------------------------------------------------
dotpro_w8_10 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_10,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_10 );

dotpro_w8_11 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_11,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_11 );

dotpro_w8_12 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_12,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_12 );

dotpro_w8_13 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_13,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_13 );

dotpro_w8_14 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_14,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_14 );

dotpro_w8_15 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_15,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_15 );

dotpro_w8_16 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_16,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_16 );

dotpro_w8_17 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_17,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_17 );

dotpro_w8_18 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_18,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_18 );

dotpro_w8_19 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_19,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_19 );

---------------------------------------------------------
dotpro_w8_20 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_20,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_20 );

dotpro_w8_21 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_21,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_21 );

dotpro_w8_22 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_22,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_22 );

dotpro_w8_23 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_23,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_23 );

dotpro_w8_24 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_24,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_24 );

dotpro_w8_25 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_25,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_25 );

dotpro_w8_26 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_26,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_26 );

dotpro_w8_27 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_27,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_27 );

dotpro_w8_28 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_28,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_28 );

dotpro_w8_29 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_29,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_29 );

---------------------------------------------------------
dotpro_w8_30 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_30,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_30 );

dotpro_w8_31 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_31,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_31 );

dotpro_w8_32 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_32,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_32 );

dotpro_w8_33 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_33,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_33 );

dotpro_w8_34 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_34,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_34 );

dotpro_w8_35 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_35,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_35 );

dotpro_w8_36 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_36,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_36 );

dotpro_w8_37 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_37,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_37 );

dotpro_w8_38 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_38,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_38 );

dotpro_w8_39 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_39,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_39 );

---------------------------------------------------------
dotpro_w8_40 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_40,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_40 );

dotpro_w8_41 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_41,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_41 );

dotpro_w8_42 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_42,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_42 );

dotpro_w8_43 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_43,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_43 );

dotpro_w8_44 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_44,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_44 );

dotpro_w8_45 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_45,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_45 );

dotpro_w8_46 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_46,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_46 );

dotpro_w8_47 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_47,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_47 );

dotpro_w8_48 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_48,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_48 );

dotpro_w8_49 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_49,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_49 );

---------------------------------------------------------
dotpro_w8_50 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_50,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_50 );

dotpro_w8_51 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_51,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_51 );

dotpro_w8_52 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_52,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_52 );

dotpro_w8_53 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_53,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_53 );

dotpro_w8_54 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_54,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_54 );

dotpro_w8_55 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_55,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_55 );

dotpro_w8_56 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_56,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_56 );

dotpro_w8_57 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_57,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_57 );

dotpro_w8_58 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_58,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_58 );

dotpro_w8_59 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_59,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_59 );

---------------------------------------------------------
dotpro_w8_60 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_60,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_60 );

dotpro_w8_61 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_61,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_61 );

dotpro_w8_62 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_62,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_62 );

dotpro_w8_63 : dotpro_w8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_63,
  CO9is1 => CO9is1,
  CO256is0 => CO256is0,
  MATMULOUT => mac_63 );

-------------------------------------------------------
  truncate_00 <= mac_00(13 downto 7);
  truncate_01 <= mac_01(13 downto 7);
  truncate_02 <= mac_02(13 downto 7);
  truncate_03 <= mac_03(13 downto 7);
  truncate_04 <= mac_04(13 downto 7);
  truncate_05 <= mac_05(13 downto 7);
  truncate_06 <= mac_06(13 downto 7);
  truncate_07 <= mac_07(13 downto 7);
  truncate_08 <= mac_08(13 downto 7);
  truncate_09 <= mac_09(13 downto 7);
  truncate_10 <= mac_10(13 downto 7);
  truncate_11 <= mac_11(13 downto 7);
  truncate_12 <= mac_12(13 downto 7);
  truncate_13 <= mac_13(13 downto 7);
  truncate_14 <= mac_14(13 downto 7);
  truncate_15 <= mac_15(13 downto 7);
  truncate_16 <= mac_16(13 downto 7);
  truncate_17 <= mac_17(13 downto 7);
  truncate_18 <= mac_18(13 downto 7);
  truncate_19 <= mac_19(13 downto 7);
  truncate_20 <= mac_20(13 downto 7);
  truncate_21 <= mac_21(13 downto 7);
  truncate_22 <= mac_22(13 downto 7);
  truncate_23 <= mac_23(13 downto 7);
  truncate_24 <= mac_24(13 downto 7);
  truncate_25 <= mac_25(13 downto 7);
  truncate_26 <= mac_26(13 downto 7);
  truncate_27 <= mac_27(13 downto 7);
  truncate_28 <= mac_28(13 downto 7);
  truncate_29 <= mac_29(13 downto 7);
  truncate_30 <= mac_30(13 downto 7);
  truncate_31 <= mac_31(13 downto 7);
  truncate_32 <= mac_32(13 downto 7);
  truncate_33 <= mac_33(13 downto 7);
  truncate_34 <= mac_34(13 downto 7);
  truncate_35 <= mac_35(13 downto 7);
  truncate_36 <= mac_36(13 downto 7);
  truncate_37 <= mac_37(13 downto 7);
  truncate_38 <= mac_38(13 downto 7);
  truncate_39 <= mac_39(13 downto 7);
  truncate_40 <= mac_40(13 downto 7);
  truncate_41 <= mac_41(13 downto 7);
  truncate_42 <= mac_42(13 downto 7);
  truncate_43 <= mac_43(13 downto 7);
  truncate_44 <= mac_44(13 downto 7);
  truncate_45 <= mac_45(13 downto 7);
  truncate_46 <= mac_46(13 downto 7);
  truncate_47 <= mac_47(13 downto 7);
  truncate_48 <= mac_48(13 downto 7);
  truncate_49 <= mac_49(13 downto 7);
  truncate_50 <= mac_50(13 downto 7);
  truncate_51 <= mac_51(13 downto 7);
  truncate_52 <= mac_52(13 downto 7);
  truncate_53 <= mac_53(13 downto 7);
  truncate_54 <= mac_54(13 downto 7);
  truncate_55 <= mac_55(13 downto 7);
  truncate_56 <= mac_56(13 downto 7);
  truncate_57 <= mac_57(13 downto 7);
  truncate_58 <= mac_58(13 downto 7);
  truncate_59 <= mac_59(13 downto 7);
  truncate_60 <= mac_60(13 downto 7);
  truncate_61 <= mac_61(13 downto 7);
  truncate_62 <= mac_62(13 downto 7);
  truncate_63 <= mac_63(13 downto 7);

---------------------------------------------------------
  Relu_00 <= (others => '0') when truncate_00 < 0 else
             (others => '1') when truncate_00 > 31 else
             truncate_00(4 downto 0);

  Relu_01 <= (others => '0') when truncate_01 < 0 else
             (others => '1') when truncate_01 > 31 else
             truncate_01(4 downto 0);

  Relu_02 <= (others => '0') when truncate_02 < 0 else
             (others => '1') when truncate_02 > 31 else
             truncate_02(4 downto 0);

  Relu_03 <= (others => '0') when truncate_03 < 0 else
             (others => '1') when truncate_03 > 31 else
             truncate_03(4 downto 0);

  Relu_04 <= (others => '0') when truncate_04 < 0 else
             (others => '1') when truncate_04 > 31 else
             truncate_04(4 downto 0);

  Relu_05 <= (others => '0') when truncate_05 < 0 else
             (others => '1') when truncate_05 > 31 else
             truncate_05(4 downto 0);

  Relu_06 <= (others => '0') when truncate_06 < 0 else
             (others => '1') when truncate_06 > 31 else
             truncate_06(4 downto 0);

  Relu_07 <= (others => '0') when truncate_07 < 0 else
             (others => '1') when truncate_07 > 31 else
             truncate_07(4 downto 0);

  Relu_08 <= (others => '0') when truncate_08 < 0 else
             (others => '1') when truncate_08 > 31 else
             truncate_08(4 downto 0);

  Relu_09 <= (others => '0') when truncate_09 < 0 else
             (others => '1') when truncate_09 > 31 else
             truncate_09(4 downto 0);

---------------------------------------------------------
  Relu_10 <= (others => '0') when truncate_10 < 0 else
             (others => '1') when truncate_10 > 31 else
             truncate_10(4 downto 0);

  Relu_11 <= (others => '0') when truncate_11 < 0 else
             (others => '1') when truncate_11 > 31 else
             truncate_11(4 downto 0);

  Relu_12 <= (others => '0') when truncate_12 < 0 else
             (others => '1') when truncate_12 > 31 else
             truncate_12(4 downto 0);

  Relu_13 <= (others => '0') when truncate_13 < 0 else
             (others => '1') when truncate_13 > 31 else
             truncate_13(4 downto 0);

  Relu_14 <= (others => '0') when truncate_14 < 0 else
             (others => '1') when truncate_14 > 31 else
             truncate_14(4 downto 0);

  Relu_15 <= (others => '0') when truncate_15 < 0 else
             (others => '1') when truncate_15 > 31 else
             truncate_15(4 downto 0);

  Relu_16 <= (others => '0') when truncate_16 < 0 else
             (others => '1') when truncate_16 > 31 else
             truncate_16(4 downto 0);

  Relu_17 <= (others => '0') when truncate_17 < 0 else
             (others => '1') when truncate_17 > 31 else
             truncate_17(4 downto 0);

  Relu_18 <= (others => '0') when truncate_18 < 0 else
             (others => '1') when truncate_18 > 31 else
             truncate_18(4 downto 0);

  Relu_19 <= (others => '0') when truncate_19 < 0 else
             (others => '1') when truncate_19 > 31 else
             truncate_19(4 downto 0);

---------------------------------------------------------
  Relu_20 <= (others => '0') when truncate_20 < 0 else
             (others => '1') when truncate_20 > 31 else
             truncate_20(4 downto 0);

  Relu_21 <= (others => '0') when truncate_21 < 0 else
             (others => '1') when truncate_21 > 31 else
             truncate_21(4 downto 0);

  Relu_22 <= (others => '0') when truncate_22 < 0 else
             (others => '1') when truncate_22 > 31 else
             truncate_22(4 downto 0);

  Relu_23 <= (others => '0') when truncate_23 < 0 else
             (others => '1') when truncate_23 > 31 else
             truncate_23(4 downto 0);

  Relu_24 <= (others => '0') when truncate_24 < 0 else
             (others => '1') when truncate_24 > 31 else
             truncate_24(4 downto 0);

  Relu_25 <= (others => '0') when truncate_25 < 0 else
             (others => '1') when truncate_25 > 31 else
             truncate_25(4 downto 0);

  Relu_26 <= (others => '0') when truncate_26 < 0 else
             (others => '1') when truncate_26 > 31 else
             truncate_26(4 downto 0);

  Relu_27 <= (others => '0') when truncate_27 < 0 else
             (others => '1') when truncate_27 > 31 else
             truncate_27(4 downto 0);

  Relu_28 <= (others => '0') when truncate_28 < 0 else
             (others => '1') when truncate_28 > 31 else
             truncate_28(4 downto 0);

  Relu_29 <= (others => '0') when truncate_29 < 0 else
             (others => '1') when truncate_29 > 31 else
             truncate_29(4 downto 0);

---------------------------------------------------------
  Relu_30 <= (others => '0') when truncate_30 < 0 else
             (others => '1') when truncate_30 > 31 else
             truncate_30(4 downto 0);

  Relu_31 <= (others => '0') when truncate_31 < 0 else
             (others => '1') when truncate_31 > 31 else
             truncate_31(4 downto 0);

  Relu_32 <= (others => '0') when truncate_32 < 0 else
             (others => '1') when truncate_32 > 31 else
             truncate_32(4 downto 0);

  Relu_33 <= (others => '0') when truncate_33 < 0 else
             (others => '1') when truncate_33 > 31 else
             truncate_33(4 downto 0);

  Relu_34 <= (others => '0') when truncate_34 < 0 else
             (others => '1') when truncate_34 > 31 else
             truncate_34(4 downto 0);

  Relu_35 <= (others => '0') when truncate_35 < 0 else
             (others => '1') when truncate_35 > 31 else
             truncate_35(4 downto 0);

  Relu_36 <= (others => '0') when truncate_36 < 0 else
             (others => '1') when truncate_36 > 31 else
             truncate_36(4 downto 0);

  Relu_37 <= (others => '0') when truncate_37 < 0 else
             (others => '1') when truncate_37 > 31 else
             truncate_37(4 downto 0);

  Relu_38 <= (others => '0') when truncate_38 < 0 else
             (others => '1') when truncate_38 > 31 else
             truncate_38(4 downto 0);

  Relu_39 <= (others => '0') when truncate_39 < 0 else
             (others => '1') when truncate_39 > 31 else
             truncate_39(4 downto 0);

---------------------------------------------------------
  Relu_40 <= (others => '0') when truncate_40 < 0 else
             (others => '1') when truncate_40 > 31 else
             truncate_40(4 downto 0);

  Relu_41 <= (others => '0') when truncate_41 < 0 else
             (others => '1') when truncate_41 > 31 else
             truncate_41(4 downto 0);

  Relu_42 <= (others => '0') when truncate_42 < 0 else
             (others => '1') when truncate_42 > 31 else
             truncate_42(4 downto 0);

  Relu_43 <= (others => '0') when truncate_43 < 0 else
             (others => '1') when truncate_43 > 31 else
             truncate_43(4 downto 0);

  Relu_44 <= (others => '0') when truncate_44 < 0 else
             (others => '1') when truncate_44 > 31 else
             truncate_44(4 downto 0);

  Relu_45 <= (others => '0') when truncate_45 < 0 else
             (others => '1') when truncate_45 > 31 else
             truncate_45(4 downto 0);

  Relu_46 <= (others => '0') when truncate_46 < 0 else
             (others => '1') when truncate_46 > 31 else
             truncate_46(4 downto 0);

  Relu_47 <= (others => '0') when truncate_47 < 0 else
             (others => '1') when truncate_47 > 31 else
             truncate_47(4 downto 0);

  Relu_48 <= (others => '0') when truncate_48 < 0 else
             (others => '1') when truncate_48 > 31 else
             truncate_48(4 downto 0);

  Relu_49 <= (others => '0') when truncate_49 < 0 else
             (others => '1') when truncate_49 > 31 else
             truncate_49(4 downto 0);

---------------------------------------------------------
  Relu_50 <= (others => '0') when truncate_50 < 0 else
             (others => '1') when truncate_50 > 31 else
             truncate_50(4 downto 0);

  Relu_51 <= (others => '0') when truncate_51 < 0 else
             (others => '1') when truncate_51 > 31 else
             truncate_51(4 downto 0);

  Relu_52 <= (others => '0') when truncate_52 < 0 else
             (others => '1') when truncate_52 > 31 else
             truncate_52(4 downto 0);

  Relu_53 <= (others => '0') when truncate_53 < 0 else
             (others => '1') when truncate_53 > 31 else
             truncate_53(4 downto 0);

  Relu_54 <= (others => '0') when truncate_54 < 0 else
             (others => '1') when truncate_54 > 31 else
             truncate_54(4 downto 0);

  Relu_55 <= (others => '0') when truncate_55 < 0 else
             (others => '1') when truncate_55 > 31 else
             truncate_55(4 downto 0);

  Relu_56 <= (others => '0') when truncate_56 < 0 else
             (others => '1') when truncate_56 > 31 else
             truncate_56(4 downto 0);

  Relu_57 <= (others => '0') when truncate_57 < 0 else
             (others => '1') when truncate_57 > 31 else
             truncate_57(4 downto 0);

  Relu_58 <= (others => '0') when truncate_58 < 0 else
             (others => '1') when truncate_58 > 31 else
             truncate_58(4 downto 0);

  Relu_59 <= (others => '0') when truncate_59 < 0 else
             (others => '1') when truncate_59 > 31 else
             truncate_59(4 downto 0);

---------------------------------------------------------
  Relu_60 <= (others => '0') when truncate_60 < 0 else
             (others => '1') when truncate_60 > 31 else
             truncate_60(4 downto 0);

  Relu_61 <= (others => '0') when truncate_61 < 0 else
             (others => '1') when truncate_61 > 31 else
             truncate_61(4 downto 0);

  Relu_62 <= (others => '0') when truncate_62 < 0 else
             (others => '1') when truncate_62 > 31 else
             truncate_62(4 downto 0);

  Relu_63 <= (others => '0') when truncate_63 < 0 else
             (others => '1') when truncate_63 > 31 else
             truncate_63(4 downto 0);



--ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
--		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
--		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
--		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
--		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)
--
--		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
--		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
--		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
--		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
--		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)
--
--		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
--		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
--		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
--		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
--		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)
--
--		+ ("0000000" & Relu_30) + ("0000000" & Relu_31);

ReluAll <= 
	reg_00_0 & reg_00_1 & reg_00_2 & reg_00_3 & reg_00_4 & reg_00_5 & reg_00_6 & reg_00_7 & 
	reg_01_0 & reg_01_1 & reg_01_2 & reg_01_3 & reg_01_4 & reg_01_5 & reg_01_6 & reg_01_7 & 
	reg_02_0 & reg_02_1 & reg_02_2 & reg_02_3 & reg_02_4 & reg_02_5 & reg_02_6 & reg_02_7 & 
	reg_03_0 & reg_03_1 & reg_03_2 & reg_03_3 & reg_03_4 & reg_03_5 & reg_03_6 & reg_03_7 & 
	reg_04_0 & reg_04_1 & reg_04_2 & reg_04_3 & reg_04_4 & reg_04_5 & reg_04_6 & reg_04_7 & 
	reg_05_0 & reg_05_1 & reg_05_2 & reg_05_3 & reg_05_4 & reg_05_5 & reg_05_6 & reg_05_7 & 
	reg_06_0 & reg_06_1 & reg_06_2 & reg_06_3 & reg_06_4 & reg_06_5 & reg_06_6 & reg_06_7 & 
	reg_07_0 & reg_07_1 & reg_07_2 & reg_07_3 & reg_07_4 & reg_07_5 & reg_07_6 & reg_07_7 & 
	reg_08_0 & reg_08_1 & reg_08_2 & reg_08_3 & reg_08_4 & reg_08_5 & reg_08_6 & reg_08_7 & 
	reg_09_0 & reg_09_1 & reg_09_2 & reg_09_3 & reg_09_4 & reg_09_5 & reg_09_6 & reg_09_7 & 
	reg_10_0 & reg_10_1 & reg_10_2 & reg_10_3 & reg_10_4 & reg_10_5 & reg_10_6 & reg_10_7 & 
	reg_11_0 & reg_11_1 & reg_11_2 & reg_11_3 & reg_11_4 & reg_11_5 & reg_11_6 & reg_11_7 & 
	reg_12_0 & reg_12_1 & reg_12_2 & reg_12_3 & reg_12_4 & reg_12_5 & reg_12_6 & reg_12_7 & 
	reg_13_0 & reg_13_1 & reg_13_2 & reg_13_3 & reg_13_4 & reg_13_5 & reg_13_6 & reg_13_7 & 
	reg_14_0 & reg_14_1 & reg_14_2 & reg_14_3 & reg_14_4 & reg_14_5 & reg_14_6 & reg_14_7 & 
	reg_15_0 & reg_15_1 & reg_15_2 & reg_15_3 & reg_15_4 & reg_15_5 & reg_15_6 & reg_15_7 & 
	reg_16_0 & reg_16_1 & reg_16_2 & reg_16_3 & reg_16_4 & reg_16_5 & reg_16_6 & reg_16_7 & 
	reg_17_0 & reg_17_1 & reg_17_2 & reg_17_3 & reg_17_4 & reg_17_5 & reg_17_6 & reg_17_7 & 
	reg_18_0 & reg_18_1 & reg_18_2 & reg_18_3 & reg_18_4 & reg_18_5 & reg_18_6 & reg_18_7 & 
	reg_19_0 & reg_19_1 & reg_19_2 & reg_19_3 & reg_19_4 & reg_19_5 & reg_19_6 & reg_19_7 & 
	reg_20_0 & reg_20_1 & reg_20_2 & reg_20_3 & reg_20_4 & reg_20_5 & reg_20_6 & reg_20_7 & 
	reg_21_0 & reg_21_1 & reg_21_2 & reg_21_3 & reg_21_4 & reg_21_5 & reg_21_6 & reg_21_7 & 
	reg_22_0 & reg_22_1 & reg_22_2 & reg_22_3 & reg_22_4 & reg_22_5 & reg_22_6 & reg_22_7 & 
	reg_23_0 & reg_23_1 & reg_23_2 & reg_23_3 & reg_23_4 & reg_23_5 & reg_23_6 & reg_23_7 & 
	reg_24_0 & reg_24_1 & reg_24_2 & reg_24_3 & reg_24_4 & reg_24_5 & reg_24_6 & reg_24_7 & 
	reg_25_0 & reg_25_1 & reg_25_2 & reg_25_3 & reg_25_4 & reg_25_5 & reg_25_6 & reg_25_7 & 
	reg_26_0 & reg_26_1 & reg_26_2 & reg_26_3 & reg_26_4 & reg_26_5 & reg_26_6 & reg_26_7 & 
	reg_27_0 & reg_27_1 & reg_27_2 & reg_27_3 & reg_27_4 & reg_27_5 & reg_27_6 & reg_27_7 & 
	reg_28_0 & reg_28_1 & reg_28_2 & reg_28_3 & reg_28_4 & reg_28_5 & reg_28_6 & reg_28_7 & 
	reg_29_0 & reg_29_1 & reg_29_2 & reg_29_3 & reg_29_4 & reg_29_5 & reg_29_6 & reg_29_7 & 
	reg_30_0 & reg_30_1 & reg_30_2 & reg_30_3 & reg_30_4 & reg_30_5 & reg_30_6 & reg_30_7 & 
	reg_31_0 & reg_31_1 & reg_31_2 & reg_31_3 & reg_31_4 & reg_31_5 & reg_31_6 & reg_31_7 & 
	reg_32_0 & reg_32_1 & reg_32_2 & reg_32_3 & reg_32_4 & reg_32_5 & reg_32_6 & reg_32_7 & 
	reg_33_0 & reg_33_1 & reg_33_2 & reg_33_3 & reg_33_4 & reg_33_5 & reg_33_6 & reg_33_7 & 
	reg_34_0 & reg_34_1 & reg_34_2 & reg_34_3 & reg_34_4 & reg_34_5 & reg_34_6 & reg_34_7 & 
	reg_35_0 & reg_35_1 & reg_35_2 & reg_35_3 & reg_35_4 & reg_35_5 & reg_35_6 & reg_35_7 & 
	reg_36_0 & reg_36_1 & reg_36_2 & reg_36_3 & reg_36_4 & reg_36_5 & reg_36_6 & reg_36_7 & 
	reg_37_0 & reg_37_1 & reg_37_2 & reg_37_3 & reg_37_4 & reg_37_5 & reg_37_6 & reg_37_7 & 
	reg_38_0 & reg_38_1 & reg_38_2 & reg_38_3 & reg_38_4 & reg_38_5 & reg_38_6 & reg_38_7 & 
	reg_39_0 & reg_39_1 & reg_39_2 & reg_39_3 & reg_39_4 & reg_39_5 & reg_39_6 & reg_39_7 & 
	reg_40_0 & reg_40_1 & reg_40_2 & reg_40_3 & reg_40_4 & reg_40_5 & reg_40_6 & reg_40_7 & 
	reg_41_0 & reg_41_1 & reg_41_2 & reg_41_3 & reg_41_4 & reg_41_5 & reg_41_6 & reg_41_7 & 
	reg_42_0 & reg_42_1 & reg_42_2 & reg_42_3 & reg_42_4 & reg_42_5 & reg_42_6 & reg_42_7 & 
	reg_43_0 & reg_43_1 & reg_43_2 & reg_43_3 & reg_43_4 & reg_43_5 & reg_43_6 & reg_43_7 & 
	reg_44_0 & reg_44_1 & reg_44_2 & reg_44_3 & reg_44_4 & reg_44_5 & reg_44_6 & reg_44_7 & 
	reg_45_0 & reg_45_1 & reg_45_2 & reg_45_3 & reg_45_4 & reg_45_5 & reg_45_6 & reg_45_7 & 
	reg_46_0 & reg_46_1 & reg_46_2 & reg_46_3 & reg_46_4 & reg_46_5 & reg_46_6 & reg_46_7 & 
	reg_47_0 & reg_47_1 & reg_47_2 & reg_47_3 & reg_47_4 & reg_47_5 & reg_47_6 & reg_47_7 & 
	reg_48_0 & reg_48_1 & reg_48_2 & reg_48_3 & reg_48_4 & reg_48_5 & reg_48_6 & reg_48_7 & 
	reg_49_0 & reg_49_1 & reg_49_2 & reg_49_3 & reg_49_4 & reg_49_5 & reg_49_6 & reg_49_7 & 
	reg_50_0 & reg_50_1 & reg_50_2 & reg_50_3 & reg_50_4 & reg_50_5 & reg_50_6 & reg_50_7 & 
	reg_51_0 & reg_51_1 & reg_51_2 & reg_51_3 & reg_51_4 & reg_51_5 & reg_51_6 & reg_51_7 & 
	reg_52_0 & reg_52_1 & reg_52_2 & reg_52_3 & reg_52_4 & reg_52_5 & reg_52_6 & reg_52_7 & 
	reg_53_0 & reg_53_1 & reg_53_2 & reg_53_3 & reg_53_4 & reg_53_5 & reg_53_6 & reg_53_7 & 
	reg_54_0 & reg_54_1 & reg_54_2 & reg_54_3 & reg_54_4 & reg_54_5 & reg_54_6 & reg_54_7 & 
	reg_55_0 & reg_55_1 & reg_55_2 & reg_55_3 & reg_55_4 & reg_55_5 & reg_55_6 & reg_55_7 & 
	reg_56_0 & reg_56_1 & reg_56_2 & reg_56_3 & reg_56_4 & reg_56_5 & reg_56_6 & reg_56_7 & 
	reg_57_0 & reg_57_1 & reg_57_2 & reg_57_3 & reg_57_4 & reg_57_5 & reg_57_6 & reg_57_7 & 
	reg_58_0 & reg_58_1 & reg_58_2 & reg_58_3 & reg_58_4 & reg_58_5 & reg_58_6 & reg_58_7 & 
	reg_59_0 & reg_59_1 & reg_59_2 & reg_59_3 & reg_59_4 & reg_59_5 & reg_59_6 & reg_59_7 & 
	reg_60_0 & reg_60_1 & reg_60_2 & reg_60_3 & reg_60_4 & reg_60_5 & reg_60_6 & reg_60_7 & 
	reg_61_0 & reg_61_1 & reg_61_2 & reg_61_3 & reg_61_4 & reg_61_5 & reg_61_6 & reg_61_7 & 
	reg_62_0 & reg_62_1 & reg_62_2 & reg_62_3 & reg_62_4 & reg_62_5 & reg_62_6 & reg_62_7 & 
	reg_63_0 & reg_63_1 & reg_63_2 & reg_63_3 & reg_63_4 & reg_63_5 & reg_63_6 & reg_63_7 ;

ReluAllWrN <= not enb_a;

end RTL;

